/*
 *  $Id$
 * 
 *  Copyright 2007 Sun Microsystems, Inc. All rights reserved.
 *  SUN PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

package javafx.ui;
import javafx.ui.Widget;
import java.lang.Math;
import java.awt.Dimension;

public class ClipPanel extends Widget {
    // private
    private attribute p: <<javax.swing.JPanel>>;

    // public
    public attribute content: Widget;
    public attribute clipWidth: Number;
    public attribute clipHeight: Number;
    public attribute fullHeight: Number;
    public attribute fullWidth: Number;
}

attribute ClipPanel.clipWidth = MAX_INT:Number;
attribute ClipPanel.clipHeight = MAX_INT:Number;

operation ClipPanel.createComponent() {
    p = new <<javax.swing.JPanel>>(false);
    var self = this;
    p.setLayout(new <<java.awt.LayoutManager>>() {

            operation removeLayoutComponent(comp:<<java.awt.Component>>) {
            }

            operation preferredLayoutSize(parent:<<java.awt.Container>>) {
                var comp = self.content.getComponent();
                var dim = comp.getPreferredSize();
                self.fullHeight = dim.height;
                self.fullWidth = dim.width;
                var res = new <<java.awt.Dimension>>(Math.min(dim.width, self.clipWidth),
                                     Math.min(dim.height, self.clipHeight));
                return res;
                                              
            }

            operation minimumLayoutSize(parent:<<java.awt.Container>>) {
                return new <<java.awt.Dimension>>(0, 0);
            }

            operation layoutContainer(parent:<<java.awt.Container>>) {
                var comp = self.content.getComponent();
                var dim = comp.getPreferredSize();
                if (dim.width < parent.getWidth()) {
                    dim.width = parent.getWidth();
                }
                if (dim.height < parent.getHeight()) {
                    dim.height = parent.getHeight();
                }
                self.fullHeight = dim.height;
                self.fullWidth = dim.width;
                var res = new Dimension(Math.min(dim.width, self.clipWidth),
                                        Math.min(dim.height, self.clipHeight));
                comp.setBounds(0, 0, 
                               res.width,
                               res.height);
            }
        });
    p.add(content.getComponent());
    p.setOpaque(false);
    return p;
}

trigger on ClipPanel.clipHeight = value {
    p.revalidate();
}

trigger on ClipPanel.clipWidth = value {
    p.revalidate();
}

trigger on ClipPanel.content = value {
    if (p <> null) {
        if (p.getComponentCount() > 0) {
            p.remove(0);
        }
        p.add(value.getComponent());
        p.invalidate();
    }
}
