/*
 *  $Id$
 * 
 *  Copyright 2007 Sun Microsystems, Inc. All rights reserved.
 *  SUN PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

package javafx.ui;
import javafx.ui.UIElement;
import java.awt.Color as AWTColor;

public class Color extends AbstractColor {
    public attribute red: Number;
    public attribute green: Number;
    public attribute blue: Number;
    public attribute opacity: Number;
    public operation Color(r:Number, g:Number, b:Number, a:Number);
    public operation interpolate(nextValue:Color, t: Number);
}


operation Color.interpolate(nextValue:Color, interp:Number) {
    if (nextValue == null) {
        return this;
    }
    var fromValue = this;
    var toValue = nextValue;
    var r1 = fromValue.red;
    var g1 = fromValue.green;
    var b1 = fromValue.blue;
    var a1 = fromValue.opacity;
    var r2 = toValue.red;
    var g2 = toValue.green;
    var b2 = toValue.blue;
    var a2 = toValue.opacity;
    var invInterp = 1.0 - interp;
    var red = r1*invInterp + r2*interp;
    var green = g1*invInterp + g2*interp ;
    var blue =   b1*invInterp + b2*interp;
    var opacity = a1*invInterp + a2*interp;
    if (red == this.red and green == this.green and blue == this.blue and opacity == this.opacity) {
        return this;
    }
    return new Color(red, green, blue, opacity);
}

attribute Color.opacity = 1.0;

trigger on Color.red = value {
    if (value > 1) {
        red = value/255;
    }
}

trigger on Color.blue = value {
    if (value > 1) {
        blue = value/255;
    }
}

trigger on Color.green = value {
    if (value > 1) {
        green = value/255;
    }
}

trigger on Color.opacity = value {
    if (value > 1) {
        opacity = value/255;
    }
}

operation Color.Color(r:Number, g:Number, b:Number, a:Number) {
    this.red = r;
    this.green = g;
    this.blue = b;
    this.opacity = a;
}

function Color.getPaint() {
    return this.getColor();
}

function Color.getColor() {
    return new AWTColor(red, green, blue, opacity);
}

rgb:Color = operation(r, g, b) {return Color {red: r/255, green: g/255, blue: b/255, opacity: 1};};
rgba:Color = operation(r, g, b, a) {return Color {red: r/255, green: g/255, blue: b/255, opacity: a/255};};

rgb:Paint = rgb:Color;
rgba:Paint = rgba:Color;
aliceblue:Color= rgb(0xF0, 0xF8, 0xFF);
antiquewhite:Color= rgb(0xFA, 0xEB, 0xD7);
aqua:Color= rgb(0x00, 0xFF, 0xFF);
aquamarine:Color= rgb(0x7F, 0xFF, 0xD4);
azure:Color= rgb(0xF0, 0xFF, 0xFF);
beige:Color= rgb(0xF5, 0xF5, 0xDC);
bisque:Color= rgb(0xFF, 0xE4, 0xC4);
black:Color= rgb(0x00, 0x00, 0x00);
blanchedalmond:Color= rgb(0xFF, 0xEB, 0xCD);
blue:Color= rgb(0x00, 0x00, 0xFF);
blueviolet:Color= rgb(0x8A, 0x2B, 0xE2);
brown:Color= rgb(0xA5, 0x2A, 0x2A);
burlywood:Color= rgb(0xDE, 0xB8, 0x87);
cadetblue:Color= rgb(0x5F, 0x9E, 0xA0);
chartreuse:Color= rgb(0x7F, 0xFF, 0x00);
chocolate:Color= rgb(0xD2, 0x69, 0x1E);
coral:Color= rgb(0xFF, 0x7F, 0x50);
cornflowerblue:Color= rgb(0x64, 0x95, 0xED);
cornsilk:Color= rgb(0xFF, 0xF8, 0xDC);
crimson:Color= rgb(0xDC, 0x14, 0x3C);
cyan:Color= rgb(0x00, 0xFF, 0xFF);
darkblue:Color= rgb(0x00, 0x00, 0x8B);
darkcyan:Color= rgb(0x00, 0x8B, 0x8B);
darkgoldenrod:Color= rgb(0xB8, 0x86, 0x0B);
darkgray:Color= rgb(0xA9, 0xA9, 0xA9);
darkgreen:Color= rgb(0x00, 0x64, 0x00);
darkkhaki:Color= rgb(0xBD, 0xB7, 0x6B);
darkmagenta:Color= rgb(0x8B, 0x00, 0x8B);
darkolivegreen:Color= rgb(0x55, 0x6B, 0x2F);
darkorange:Color= rgb(0xFF, 0x8C, 0x00);
darkorchid:Color= rgb(0x99, 0x32, 0xCC);
darkred:Color= rgb(0x8B, 0x00, 0x00);
darksalmon:Color= rgb(0xE9, 0x96, 0x7A);
darkseagreen:Color= rgb(0x8F, 0xBC, 0x8B);
darkslateblue:Color= rgb(0x48, 0x3D, 0x8B);
darkslategray:Color= rgb(0x2F, 0x4F, 0x4F);
darkturquoise:Color= rgb(0x00, 0xCE, 0xD1);
darkviolet:Color= rgb(0x94, 0x00, 0xD3);
deeppink:Color= rgb(0xFF, 0x14, 0x93);
deepskyblue:Color= rgb(0x00, 0xBF, 0xFF);
dimgray:Color= rgb(0x69, 0x69, 0x69);
dodgerblue:Color= rgb(0x1E, 0x90, 0xFF);
firebrick:Color= rgb(0xB2, 0x22, 0x22);
floralwhite:Color= rgb(0xFF, 0xFA, 0xF0);
forestgreen:Color= rgb(0x22, 0x8B, 0x22);
fuchsia:Color= rgb(0xFF, 0x00, 0xFF);
gainsboro:Color= rgb(0xDC, 0xDC, 0xDC);
ghostwhite:Color= rgb(0xF8, 0xF8, 0xFF);
gold:Color= rgb(0xFF, 0xD7, 0x00);
goldenrod:Color= rgb(0xDA, 0xA5, 0x20);
gray:Color= rgb(0x80, 0x80, 0x80);
green:Color= rgb(0x00, 0x80, 0x00);
greenyellow:Color= rgb(0xAD, 0xFF, 0x2F);
honeydew:Color= rgb(0xF0, 0xFF, 0xF0);
hotpink:Color= rgb(0xFF, 0x69, 0xB4);
indianred:Color= rgb(0xCD, 0x5C, 0x5C);
indigo:Color= rgb(0x4B, 0x00, 0x82);
ivory:Color= rgb(0xFF, 0xFF, 0xF0);
khaki:Color= rgb(0xF0, 0xE6, 0x8C);
lavender:Color= rgb(0xE6, 0xE6, 0xFA);
lavenderblush:Color= rgb(0xFF, 0xF0, 0xF5);
lawngreen:Color= rgb(0x7C, 0xFC, 0x00);
lemonchiffon:Color= rgb(0xFF, 0xFA, 0xCD);
lightblue:Color= rgb(0xAD, 0xD8, 0xE6);
lightcoral:Color= rgb(0xF0, 0x80, 0x80);
lightcyan:Color= rgb(0xE0, 0xFF, 0xFF);
lightgoldenrodyellow:Color= rgb(0xFA, 0xFA, 0xD2);
lightgreen:Color= rgb(0x90, 0xEE, 0x90);
grey:Color= rgb(0x80, 0x80, 0x80);
lightgrey:Color= rgb(0xD3, 0xD3, 0xD3);
lightpink:Color= rgb(0xFF, 0xB6, 0xC1);
lightsalmon:Color= rgb(0xFF, 0xA0, 0x7A);
lightseagreen:Color= rgb(0x20, 0xB2, 0xAA);
lightskyblue:Color= rgb(0x87, 0xCE, 0xFA);
lightslategray:Color= rgb(0x77, 0x88, 0x99);
lightsteelblue:Color= rgb(0xB0, 0xC4, 0xDE);
lightyellow:Color= rgb(0xFF, 0xFF, 0xE0);
lime:Color= rgb(0x00, 0xFF, 0x00);
limegreen:Color= rgb(0x32, 0xCD, 0x32);
linen:Color= rgb(0xFA, 0xF0, 0xE6);
magenta:Color= rgb(0xFF, 0x00, 0xFF);
maroon:Color= rgb(0x80, 0x00, 0x00);
mediumaquamarine:Color= rgb(0x66, 0xCD, 0xAA);
mediumblue:Color= rgb(0x00, 0x00, 0xCD);
mediumorchid:Color= rgb(0xBA, 0x55, 0xD3);
mediumpurple:Color= rgb(0x93, 0x70, 0xDB);
mediumseagreen:Color= rgb(0x3C, 0xB3, 0x71);
mediumslateblue:Color= rgb(0x7B, 0x68, 0xEE);
mediumspringgreen:Color= rgb(0x00, 0xFA, 0x9A);
mediumturquoise:Color= rgb(0x48, 0xD1, 0xCC);
mediumvioletred:Color= rgb(0xC7, 0x15, 0x85);
midnightblue:Color= rgb(0x19, 0x19, 0x70);
mintcream:Color= rgb(0xF5, 0xFF, 0xFA);
mistyrose:Color= rgb(0xFF, 0xE4, 0xE1);
moccasin:Color= rgb(0xFF, 0xE4, 0xB5);
navajowhite:Color= rgb(0xFF, 0xDE, 0xAD);
navy:Color= rgb(0x00, 0x00, 0x80);
oldlace:Color= rgb(0xFD, 0xF5, 0xE6);
olive:Color= rgb(0x80, 0x80, 0x00);
olivedrab:Color= rgb(0x6B, 0x8E, 0x23);
orange:Color= rgb(0xFF, 0xA5, 0x00);
orangered:Color= rgb(0xFF, 0x45, 0x00);
orchid:Color= rgb(0xDA, 0x70, 0xD6);
palegoldenrod:Color= rgb(0xEE, 0xE8, 0xAA);
palegreen:Color= rgb(0x98, 0xFB, 0x98);
paleturquoise:Color= rgb(0xAF, 0xEE, 0xEE);
palevioletred:Color= rgb(0xDB, 0x70, 0x93);
papayawhip:Color= rgb(0xFF, 0xEF, 0xD5);
peachpuff:Color= rgb(0xFF, 0xDA, 0xB9);
peru:Color= rgb(0xCD, 0x85, 0x3F);
pink:Color= rgb(0xFF, 0xC0, 0xCB);
plum:Color= rgb(0xDD, 0xA0, 0xDD);
powderblue:Color= rgb(0xB0, 0xE0, 0xE6);
purple:Color= rgb(0x80, 0x00, 0x80);
red:Color= rgb(0xFF, 0x00, 0x00);
rosybrown:Color= rgb(0xBC, 0x8F, 0x8F);
royalblue:Color= rgb(0x41, 0x69, 0xE1);
saddlebrown:Color= rgb(0x8B, 0x45, 0x13);
salmon:Color= rgb(0xFA, 0x80, 0x72);
sandybrown:Color= rgb(0xF4, 0xA4, 0x60);
seagreen:Color= rgb(0x2E, 0x8B, 0x57);
seashell:Color= rgb(0xFF, 0xF5, 0xEE);
sienna:Color= rgb(0xA0, 0x52, 0x2D);
silver:Color= rgb(0xC0, 0xC0, 0xC0);
skyblue:Color= rgb(0x87, 0xCE, 0xEB);
slateblue:Color= rgb(0x6A, 0x5A, 0xCD);
slategray:Color= rgb(0x70, 0x80, 0x90);
snow:Color= rgb(0xFF, 0xFA, 0xFA);
springgreen:Color= rgb(0x00, 0xFF, 0x7F);
steelblue:Color= rgb(0x46, 0x82, 0xB4);
tan:Color= rgb(0xD2, 0xB4, 0x8C);
teal:Color= rgb(0x00, 0x80, 0x80);
thistle:Color= rgb(0xD8, 0xBF, 0xD8);
tomato:Color= rgb(0xFF, 0x63, 0x47);
turquoise:Color= rgb(0x40, 0xE0, 0xD0);
violet:Color= rgb(0xEE, 0x82, 0xEE);
wheat:Color= rgb(0xF5, 0xDE, 0xB3);
white:Color= rgb(0xFF, 0xFF, 0xFF);
whitesmoke:Color= rgb(0xF5, 0xF5, 0xF5);
yellow:Color= rgb(0xFF, 0xFF, 0x00);
yellowgreen:Color= rgb(0x9A, 0xCD, 0x32);


aliceblue:Paint= rgb(0xF0, 0xF8, 0xFF);
antiquewhite:Paint= rgb(0xFA, 0xEB, 0xD7);
aqua:Paint= rgb(0x00, 0xFF, 0xFF);
aquamarine:Paint= rgb(0x7F, 0xFF, 0xD4);
azure:Paint= rgb(0xF0, 0xFF, 0xFF);
beige:Paint= rgb(0xF5, 0xF5, 0xDC);
bisque:Paint= rgb(0xFF, 0xE4, 0xC4);
black:Paint= rgb(0x00, 0x00, 0x00);
blanchedalmond:Paint= rgb(0xFF, 0xEB, 0xCD);
blue:Paint= rgb(0x00, 0x00, 0xFF);
blueviolet:Paint= rgb(0x8A, 0x2B, 0xE2);
brown:Paint= rgb(0xA5, 0x2A, 0x2A);
burlywood:Paint= rgb(0xDE, 0xB8, 0x87);
cadetblue:Paint= rgb(0x5F, 0x9E, 0xA0);
chartreuse:Paint= rgb(0x7F, 0xFF, 0x00);
chocolate:Paint= rgb(0xD2, 0x69, 0x1E);
coral:Paint= rgb(0xFF, 0x7F, 0x50);
cornflowerblue:Paint= rgb(0x64, 0x95, 0xED);
cornsilk:Paint= rgb(0xFF, 0xF8, 0xDC);
crimson:Paint= rgb(0xDC, 0x14, 0x3C);
cyan:Paint= rgb(0x00, 0xFF, 0xFF);
darkblue:Paint= rgb(0x00, 0x00, 0x8B);
darkcyan:Paint= rgb(0x00, 0x8B, 0x8B);
darkgoldenrod:Paint= rgb(0xB8, 0x86, 0x0B);
darkgray:Paint= rgb(0xA9, 0xA9, 0xA9);
darkgreen:Paint= rgb(0x00, 0x64, 0x00);
darkkhaki:Paint= rgb(0xBD, 0xB7, 0x6B);
darkmagenta:Paint= rgb(0x8B, 0x00, 0x8B);
darkolivegreen:Paint= rgb(0x55, 0x6B, 0x2F);
darkorange:Paint= rgb(0xFF, 0x8C, 0x00);
darkorchid:Paint= rgb(0x99, 0x32, 0xCC);
darkred:Paint= rgb(0x8B, 0x00, 0x00);
darksalmon:Paint= rgb(0xE9, 0x96, 0x7A);
darkseagreen:Paint= rgb(0x8F, 0xBC, 0x8B);
darkslateblue:Paint= rgb(0x48, 0x3D, 0x8B);
darkslategray:Paint= rgb(0x2F, 0x4F, 0x4F);
darkturquoise:Paint= rgb(0x00, 0xCE, 0xD1);
darkviolet:Paint= rgb(0x94, 0x00, 0xD3);
deeppink:Paint= rgb(0xFF, 0x14, 0x93);
deepskyblue:Paint= rgb(0x00, 0xBF, 0xFF);
dimgray:Paint= rgb(0x69, 0x69, 0x69);
dodgerblue:Paint= rgb(0x1E, 0x90, 0xFF);
firebrick:Paint= rgb(0xB2, 0x22, 0x22);
floralwhite:Paint= rgb(0xFF, 0xFA, 0xF0);
forestgreen:Paint= rgb(0x22, 0x8B, 0x22);
fuchsia:Paint= rgb(0xFF, 0x00, 0xFF);
gainsboro:Paint= rgb(0xDC, 0xDC, 0xDC);
ghostwhite:Paint= rgb(0xF8, 0xF8, 0xFF);
gold:Paint= rgb(0xFF, 0xD7, 0x00);
goldenrod:Paint= rgb(0xDA, 0xA5, 0x20);
gray:Paint= rgb(0x80, 0x80, 0x80);
green:Paint= rgb(0x00, 0x80, 0x00);
greenyellow:Paint= rgb(0xAD, 0xFF, 0x2F);
honeydew:Paint= rgb(0xF0, 0xFF, 0xF0);
hotpink:Paint= rgb(0xFF, 0x69, 0xB4);
indianred:Paint= rgb(0xCD, 0x5C, 0x5C);
indigo:Paint= rgb(0x4B, 0x00, 0x82);
ivory:Paint= rgb(0xFF, 0xFF, 0xF0);
khaki:Paint= rgb(0xF0, 0xE6, 0x8C);
lavender:Paint= rgb(0xE6, 0xE6, 0xFA);
lavenderblush:Paint= rgb(0xFF, 0xF0, 0xF5);
lawngreen:Paint= rgb(0x7C, 0xFC, 0x00);
lemonchiffon:Paint= rgb(0xFF, 0xFA, 0xCD);
lightblue:Paint= rgb(0xAD, 0xD8, 0xE6);
lightcoral:Paint= rgb(0xF0, 0x80, 0x80);
lightcyan:Paint= rgb(0xE0, 0xFF, 0xFF);
lightgoldenrodyellow:Paint= rgb(0xFA, 0xFA, 0xD2);
lightgreen:Paint= rgb(0x90, 0xEE, 0x90);
grey:Paint= rgb(0x80, 0x80, 0x80);
lightgrey:Paint= rgb(0xD3, 0xD3, 0xD3);
lightpink:Paint= rgb(0xFF, 0xB6, 0xC1);
lightsalmon:Paint= rgb(0xFF, 0xA0, 0x7A);
lightseagreen:Paint= rgb(0x20, 0xB2, 0xAA);
lightskyblue:Paint= rgb(0x87, 0xCE, 0xFA);
lightslategray:Paint= rgb(0x77, 0x88, 0x99);
lightsteelblue:Paint= rgb(0xB0, 0xC4, 0xDE);
lightyellow:Paint= rgb(0xFF, 0xFF, 0xE0);
lime:Paint= rgb(0x00, 0xFF, 0x00);
limegreen:Paint= rgb(0x32, 0xCD, 0x32);
linen:Paint= rgb(0xFA, 0xF0, 0xE6);
magenta:Paint= rgb(0xFF, 0x00, 0xFF);
maroon:Paint= rgb(0x80, 0x00, 0x00);
mediumaquamarine:Paint= rgb(0x66, 0xCD, 0xAA);
mediumblue:Paint= rgb(0x00, 0x00, 0xCD);
mediumorchid:Paint= rgb(0xBA, 0x55, 0xD3);
mediumpurple:Paint= rgb(0x93, 0x70, 0xDB);
mediumseagreen:Paint= rgb(0x3C, 0xB3, 0x71);
mediumslateblue:Paint= rgb(0x7B, 0x68, 0xEE);
mediumspringgreen:Paint= rgb(0x00, 0xFA, 0x9A);
mediumturquoise:Paint= rgb(0x48, 0xD1, 0xCC);
mediumvioletred:Paint= rgb(0xC7, 0x15, 0x85);
midnightblue:Paint= rgb(0x19, 0x19, 0x70);
mintcream:Paint= rgb(0xF5, 0xFF, 0xFA);
mistyrose:Paint= rgb(0xFF, 0xE4, 0xE1);
moccasin:Paint= rgb(0xFF, 0xE4, 0xB5);
navajowhite:Paint= rgb(0xFF, 0xDE, 0xAD);
navy:Paint= rgb(0x00, 0x00, 0x80);
oldlace:Paint= rgb(0xFD, 0xF5, 0xE6);
olive:Paint= rgb(0x80, 0x80, 0x00);
olivedrab:Paint= rgb(0x6B, 0x8E, 0x23);
orange:Paint= rgb(0xFF, 0xA5, 0x00);
orangered:Paint= rgb(0xFF, 0x45, 0x00);
orchid:Paint= rgb(0xDA, 0x70, 0xD6);
palegoldenrod:Paint= rgb(0xEE, 0xE8, 0xAA);
palegreen:Paint= rgb(0x98, 0xFB, 0x98);
paleturquoise:Paint= rgb(0xAF, 0xEE, 0xEE);
palevioletred:Paint= rgb(0xDB, 0x70, 0x93);
papayawhip:Paint= rgb(0xFF, 0xEF, 0xD5);
peachpuff:Paint= rgb(0xFF, 0xDA, 0xB9);
peru:Paint= rgb(0xCD, 0x85, 0x3F);
pink:Paint= rgb(0xFF, 0xC0, 0xCB);
plum:Paint= rgb(0xDD, 0xA0, 0xDD);
powderblue:Paint= rgb(0xB0, 0xE0, 0xE6);
purple:Paint= rgb(0x80, 0x00, 0x80);
red:Paint= rgb(0xFF, 0x00, 0x00);
rosybrown:Paint= rgb(0xBC, 0x8F, 0x8F);
royalblue:Paint= rgb(0x41, 0x69, 0xE1);
saddlebrown:Paint= rgb(0x8B, 0x45, 0x13);
salmon:Paint= rgb(0xFA, 0x80, 0x72);
sandybrown:Paint= rgb(0xF4, 0xA4, 0x60);
seagreen:Paint= rgb(0x2E, 0x8B, 0x57);
seashell:Paint= rgb(0xFF, 0xF5, 0xEE);
sienna:Paint= rgb(0xA0, 0x52, 0x2D);
silver:Paint= rgb(0xC0, 0xC0, 0xC0);
skyblue:Paint= rgb(0x87, 0xCE, 0xEB);
slateblue:Paint= rgb(0x6A, 0x5A, 0xCD);
slategray:Paint= rgb(0x70, 0x80, 0x90);
snow:Paint= rgb(0xFF, 0xFA, 0xFA);
springgreen:Paint= rgb(0x00, 0xFF, 0x7F);
steelblue:Paint= rgb(0x46, 0x82, 0xB4);
tan:Paint= rgb(0xD2, 0xB4, 0x8C);
teal:Paint= rgb(0x00, 0x80, 0x80);
thistle:Paint= rgb(0xD8, 0xBF, 0xD8);
tomato:Paint= rgb(0xFF, 0x63, 0x47);
turquoise:Paint= rgb(0x40, 0xE0, 0xD0);
violet:Paint= rgb(0xEE, 0x82, 0xEE);
wheat:Paint= rgb(0xF5, 0xDE, 0xB3);
white:Paint = rgb(0xFF, 0xFF, 0xFF);
whitesmoke:Paint= rgb(0xF5, 0xF5, 0xF5);
yellow:Paint= rgb(0xFF, 0xFF, 0x00);
yellowgreen:Paint= rgb(0x9A, 0xCD, 0x32);     

fromAWTColor:Color = function(c:AWTColor) = Color {red: c.getRed()/255, green: c.getGreen()/255, blue: c.getBlue()/ 255, opacity: c.getAlpha()/255};
transparent:Color = function(c:Color, opacity:Number) = new Color(c.red, c.green, c.blue, opacity);

lightGray:Color = fromAWTColor(AWTColor.lightGray);
lightGray:Paint = lightGray:Color;

animateColor:Paint = function(colors:Color*, millis:Number, repeat:Boolean){
    var t = unitinterval in dur millis fps 10 continue if repeat;
    operation interpolateColor(colors:Color*, t:Number) {
        var off = (t * (sizeof colors - 1));
        var i = off.intValue();
        var color1 = colors[i];
        var color2 = colors[i+1];
        return color1.interpolate(color2, off - i);
    }
    return interpolateColor(colors, t);
};


operation __INTERPOLATE_COLOR(color1:Color, color2:Color, t:Number) {
    return color1.interpolate(color2, t);
}
    

EASEBOTH:Color = operation(a:Color*, t:Number) {
     return __EASE(a, t, __EASEBOTH, __INTERPOLATE_COLOR);
};

EASEIN:Color = operation(a:Color*, t:Number) {
     return __EASE(a, t, __EASEIN, __INTERPOLATE_COLOR);
};

EASEOUT:Color = operation(a:Color*, t:Number) {
     return __EASE(a, t, __EASEOUT, __INTERPOLATE_COLOR);
};

LINEAR:Color = operation(a:Color*, t:Number) {
        return __EASE(a, t, function(t:Number) {return t;}, __INTERPOLATE_COLOR);
};

DISCRETE:Color = operation(a:Color*, t:Number) {
    return a[t*(sizeof a -1)];
};


