/*
 *  $Id$
 * 
 *  Copyright 2007 Sun Microsystems, Inc. All rights reserved.
 *  SUN PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

package javafx.ui;
import javafx.ui.Widget;

// GridLayout

public class GridPanel extends Widget {
    private attribute jpanel: <<javax.swing.JPanel>>;
    private attribute layout: <<java.awt.GridLayout>>;

    public attribute rows: Number;
    public attribute columns: Number;
    public attribute hgap: Number?;
    public attribute vgap: Number?;
    public attribute cells: Widget*;
}

trigger on (GridPanel.hgap = value) {
    layout.setHgap(value);
}

trigger on (GridPanel.vgap = value) {
    layout.setVgap(value);
}

trigger on (GridPanel.columns = value) {
    layout.setColumns(value);
}

trigger on (GridPanel.rows = value) {
    layout.setRows(value);
}

operation GridPanel.createComponent():<<javax.swing.JComponent>> {
    jpanel = new <<javax.swing.JPanel>>();
    jpanel.setOpaque(false);
    layout = new <<java.awt.GridLayout>>(rows, columns);
    if (vgap <> null) {
        layout.setVgap(vgap);
    }
    if (hgap <> null) {
        layout.setHgap(hgap);
    }
    jpanel.setLayout(layout);
    for (i in cells) {
        jpanel.add(i.getComponent());
    }
    return jpanel;
}

trigger on (insert cell into GridPanel.cells) {
    if (jpanel <> null) {
        jpanel.add(cell.getComponent(), indexof cell);
        jpanel.validate();
    }
}

trigger on (delete oldCell from GridPanel.cells) {
    if (jpanel <> null) {
        jpanel.remove(indexof oldCell);
        jpanel.validate();
    }
}
