/*
 *  $Id$
 * 
 *  Copyright 2007 Sun Microsystems, Inc. All rights reserved.
 *  SUN PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

package javafx.ui;
import javafx.ui.Widget;
import javafx.ui.Orientation;
import javafx.ui.SliderLabel;

public class Slider extends Widget {
    private attribute jslider:<<javax.swing.JSlider>>;

    public attribute min: Number;
    public attribute max: Number;
    public attribute value: Number;
    public attribute extent: Number?;
    public attribute orientation: Orientation;
    public attribute majorTickSpacing: Number;
    public attribute minorTickSpacing: Number;
    public attribute snapToTicks: Boolean;
    public attribute paintTicks: Boolean;
    public attribute paintTrack: Boolean;
    public attribute paintLabels: Boolean;
    public attribute labels: SliderLabel*;
    public attribute filled: Boolean;
}

trigger on (new Slider) {
    paintTrack = true;
    paintLabels = false;
    paintTicks = false;
    majorTickSpacing = 0;
    minorTickSpacing = 0;
    snapToTicks = false;
    filled = false;
    orientation = HORIZONTAL:Orientation;
    min = 0;
    max = 100;
    value = 50;
}

trigger on (Slider.paintTrack = value) {
    jslider.setPaintTrack(value);
}

trigger on (Slider.min = value) {
    jslider.setMinimum(value);
}

trigger on (Slider.max = value) {
    jslider.setMaximum(value);
}

trigger on (Slider.value = newValue) {
    jslider.setValue(newValue);
}

trigger on (Slider.orientation = value) {
    jslider.setOrientation(if (value == VERTICAL:Orientation) 
                               then jslider.VERTICAL
                               else jslider.HORIZONTAL);
}

trigger on (Slider.snapToTicks = value) {
    jslider.setSnapToTicks(value);
}


trigger on (Slider.paintTicks = value) {
    jslider.setPaintTicks(value);
}

trigger on (Slider.paintLabels = value) {
    jslider.setPaintLabels(value);
}

trigger on (Slider.majorTickSpacing = value) {
    jslider.setMajorTickSpacing(value);
}

trigger on (Slider.minorTickSpacing = value) {
    jslider.setMinorTickSpacing(value);
}

trigger on (insert label into Slider.labels) {
    var dict = new <<java.util.Hashtable>>();
    for (i in labels) {
        dict.put(i.value.intValue(), i.label.getComponent());
    }
    paintLabels = true;
    jslider.setLabelTable(dict);
}

trigger on (delete label from Slider.labels) {
    var dict = new <<java.util.Hashtable>>();
    for (i in labels) {
        dict.put(i.value.intValue(), i.label.getComponent());
    }
    jslider.setLabelTable(dict);
}

trigger on (Slider.filled = value) {
    jslider.putClientProperty("JSlider.isFilled", value);
}
operation Slider.createComponent() {
    try {
        jslider = new <<javax.swing.JSlider>>(if (orientation == VERTICAL:Orientation) then <<javax.swing.JSlider>>.VERTICAL else <<javax.swing.JSlider>>.HORIZONTAL,
                                          min, max, value);
    } catch (e) {
        throw "{e} min = {min} max={max} value={value}";
    }
    jslider.setOpaque(false);
    jslider.setSnapToTicks(snapToTicks);
    jslider.setPaintTicks(paintTicks);
    jslider.setPaintLabels(paintLabels);
    jslider.setMajorTickSpacing(majorTickSpacing);
    jslider.setMinorTickSpacing(minorTickSpacing);
    jslider.setPaintTrack(paintTrack);
    if (extent <> null) {
        jslider.setExtent(extent);
    }
    if (sizeof labels > 0) {
        var dict = new <<java.util.Hashtable>>();
        for (i in labels) {
            dict.put(i.value.intValue(), i.label.getComponent());
        }
        jslider.setLabelTable(dict);
    }
    if (filled) {
        jslider.putClientProperty("JSlider.isFilled", true);
    }
    var self = this;
    jslider.addChangeListener(new <<javax.swing.event.ChangeListener>>() {
                                  operation stateChanged(e) {
                                      self.value = self.jslider.getValue();
                                  }
                              });
    return jslider;
}
