/*
 *  $Id$
 * 
 *  Copyright 2007 Sun Microsystems, Inc. All rights reserved.
 *  SUN PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

package javafx.ui;
import javafx.ui.Widget;
import javafx.ui.Tab;
import javafx.ui.SingleSelection;
import javafx.ui.TabPlacement;
import javafx.ui.TabLayout;
import javafx.ui.WidgetInitiatedMultiSelection;

public class TabbedPane extends Widget {
    protected attribute tabbedpane: <<javax.swing.JTabbedPane>>;
    private attribute changeListener: <<javax.swing.event.ChangeListener>>;
    public attribute selectedIndex: Number?;
    public attribute onSelectionChange: function(oldSelection:SingleSelection, newSelection:SingleSelection);
    public attribute selection: SingleSelection?;
    public attribute tabPlacement: TabPlacement?;
    public attribute tabLayout: TabLayout?;
    public attribute tabs: Tab*; 
}

attribute TabbedPane.opaque = true;

trigger on (TabbedPane.tabLayout = layout) {
    if (layout == WRAP:TabLayout) {
        tabbedpane.setTabLayoutPolicy(tabbedpane.WRAP_TAB_LAYOUT);
    } else {
        tabbedpane.setTabLayoutPolicy(tabbedpane.SCROLL_TAB_LAYOUT);
    }
}

trigger on (TabbedPane.tabPlacement = value) {
    if (value <> null) {
        tabbedpane.setTabPlacement(value.id);
    } 
}

operation TabbedPane.createComponent():<<javax.swing.JComponent>> {
    if (selectedIndex >= 0 and selectedIndex < tabbedpane.getTabCount()) {
        for (i in tabs) {
            i.selectTab();
        }
        tabbedpane.setSelectedIndex(selectedIndex);
    }
    return tabbedpane;
}

trigger on (new TabbedPane) {
    tabbedpane = context:<<net.java.javafx.ui.UIContext>>.createTabbedPane();
    tabbedpane.setOpaque(false);
    var self = this;
    selectedIndex = 0;
    changeListener = new <<javax.swing.event.ChangeListener>>() {
        operation stateChanged(e:<<javax.swing.event.ChangeEvent>>) {
            self.selection = new WidgetInitiatedMultiSelection {
                    widget: self.tabbedpane,
                    anchorIndex: self.tabbedpane.getSelectedIndex()
           };
        }
    };
    tabbedpane.getModel().addChangeListener(changeListener);
}

trigger on (TabbedPane.selectedIndex = n) {
    if (component == null) {
        return; // hack
    }
    var model = tabbedpane.getModel();
    model.removeChangeListener(changeListener);
    if (n == null) {
        n = -1;
    }
    var tab = tabs[n];
    if (tab <> null) {
        tab.selectTab();
        model.setSelectedIndex(n);
    }
    model.addChangeListener(changeListener);
}

trigger on (TabbedPane.selection[old] = value) {
    if (component <> null) {
        selectedIndex = value.anchorIndex;
        tabs[value.anchorIndex].selectTab();
        (this.onSelectionChange)(old, value);
    }
}

trigger on (insert tab into TabbedPane.tabs) {
    tab.tabbedPane = this;
    var icon:<<javax.swing.Icon>> = null;
    if (tab.icon <> null) {
        icon = tab.icon.getIcon();
    }
    tabbedpane.insertTab(tab.title, icon, tab.panel, tab.toolTipText, indexof tab);
    if (indexof tab == selectedIndex) {
        tabbedpane.setSelectedIndex(selectedIndex);
        selection = new SingleSelection {anchorIndex: selectedIndex};
    }
}

trigger on (delete tab from TabbedPane.tabs) {
    tabbedpane.removeTabAt(indexof tab);
}

trigger on TabbedPane.tabs[old] = tab {
    tab.tabbedPane = this;
    var icon:<<javax.swing.Icon>> = null;
    if (tab.icon <> null) {
        icon = tab.icon.getIcon();
    }
    var i = indexof tab;
    tabbedpane.setTitleAt(i, tab.title);
    tabbedpane.setIconAt(i, icon);
    tabbedpane.setToolTipTextAt(i, tab.toolTipText);
    tabbedpane.setComponentAt(i, tab.panel);
}
