/*
 *  $Id$
 * 
 *  Copyright 2007 Sun Microsystems, Inc. All rights reserved.
 *  SUN PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

package javafx.ui;
import javafx.ui.ScrollableWidget;
import javax.swing.event.CaretListener;
import javax.swing.event.CaretEvent;
import java.awt.Point;
import java.awt.Rectangle;
import javax.swing.JTextArea;

public class TextArea extends ScrollableWidget {
    private attribute caretColor: Color?;
    private attribute inUpdate:Boolean;
    private attribute documentListener:<<javax.swing.event.DocumentListener>>;
    private attribute jtextarea: JTextArea;
    public attribute editable: Boolean?;
    public attribute rows: Number?;
    public attribute columns: Number?;
    public attribute lineWrap: Boolean?;
    public attribute tabSize: Number?;
    public attribute wrapStyleWord: Boolean?;
    public attribute text:String;
    public attribute caretDot: Integer;
    public attribute caretMark: Integer;
    public attribute lineCount: Number;
    public operation modelToView(offset:Integer): Rectangle;
    public operation viewToModel(p:Point): Number;
    public operation getLineOfOffset(offset:Integer): Integer;
    public operation getLineStartOffset(line:Integer): Integer;
    public operation replaceRange(replacement:String, start:Number, end:Number);
    public operation setCaretPosition(pos:Number);
}

attribute TextArea.caretColor = bind foreground;

trigger on TextArea.caretColor = value {
    jtextarea.setCaretColor(value.getColor());
}

operation TextArea.setCaretPosition(pos:Number) {
    jtextarea.setCaretPosition(pos);
}

operation TextArea.modelToView(offset:Integer) {
    return jtextarea.modelToView(offset);
}

operation TextArea.viewToModel(p:Point) {
    return jtextarea.viewToModel(p);
}

trigger on TextArea.tabSize = value {
    jtextarea.setTabSize(value);
}

operation TextArea.getLineOfOffset(offset:Integer) {
    return jtextarea.getLineOfOffset(offset);
}

operation TextArea.getLineStartOffset(line: Number) {
    return jtextarea.getLineStartOffset(line);
}

operation TextArea.createView():<<javax.swing.JComponent>> {
    if (editable) {
        jtextarea.<<select>>(0,0);
    }
    var self = this;
    documentListener = new <<javax.swing.event.DocumentListener>>() {
        operation insertUpdate(e:<<javax.swing.event.DocumentEvent>>) {
            self.inUpdate = true;
            self.text = self.jtextarea.getText();
            do later {
                self.lineCount = self.jtextarea.getLineCount();
            }
            self.inUpdate = false;
        }
        operation removeUpdate(e:<<javax.swing.event.DocumentEvent>>) {
            self.inUpdate = true;
            self.text = self.jtextarea.getText();
            do later {
                self.lineCount = self.jtextarea.getLineCount();
            }
            self.inUpdate = false;
        }
        operation changedUpdate(e:<<javax.swing.event.DocumentEvent>>) {
            self.inUpdate = true;
            self.text = self.jtextarea.getText();
            do later {
                self.lineCount = self.jtextarea.getLineCount();
            }
            self.inUpdate = false;
        }
    };
    jtextarea.addCaretListener(new CaretListener() {
                operation caretUpdate(e:CaretEvent) {
                    self.caretDot = e.getDot();
                    self.caretMark = e.getMark();
                }
        });
    jtextarea.getDocument().addDocumentListener(documentListener);
    return jtextarea;
}

operation TextArea.replaceRange(value:String, start:Number, end:Number) {
    jtextarea.replaceRange(value, start, end);
    jtextarea.revalidate();
}


trigger on (TextArea.rows = value) {
    jtextarea.setRows(value);
}

trigger on (TextArea.columns = value) {
    jtextarea.setColumns(value);
}

trigger on (TextArea.lineWrap = value) {
    jtextarea.setLineWrap(value);
}

trigger on (TextArea.wrapStyleWord = value) {
    jtextarea.setWrapStyleWord(value);
}

trigger on (TextArea.editable = value) {
    jtextarea.setEditable(value);
}

trigger on (new TextArea) {
    jtextarea = new JTextArea();
    editable = true;
}

trigger on (TextArea.text = value) {
    if (not inUpdate) {
        jtextarea.setText(value);
        jtextarea.<<select>>(0, 0);
        jtextarea.revalidate();
    }
}
