/*
 *  $Id$
 * 
 *  Copyright 2007 Sun Microsystems, Inc. All rights reserved.
 *  SUN PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

package javafx.ui;
import javafx.ui.Widget;
import javafx.ui.HorizontalAlignment;

public class TextField extends Widget {
    private attribute propertyChangeListener:<<java.beans.PropertyChangeListener>>;
    private attribute textField: <<javax.swing.JTextField>>;
    private attribute verifier: <<net.java.javafx.ui.XInputVerifierImpl>>;


    public attribute value: String;
    public attribute text: String;
    public attribute onChange: function(newValue:String)?;
    public attribute action: function()?;
    public attribute columns: Number;
    public attribute editable: Boolean;
    public attribute verify: function(newValue:String):Boolean?;
    public attribute horizontalAlignment:HorizontalAlignment;
    public attribute margin: Insets?;
}

trigger on TextField.margin = value {
    textField.setMargin(value.awtinsets);
}

trigger on (TextField.horizontalAlignment = value) {
    textField.setHorizontalAlignment(value.id);
}

operation TextField.createComponent():<<javax.swing.JComponent>> {
    var self = this;
    var documentListener = new <<javax.swing.event.DocumentListener>>() {
        operation insertUpdate(e:<<javax.swing.event.DocumentEvent>>) {
            self.text = self.textField.getText();
        }
        operation removeUpdate(e:<<javax.swing.event.DocumentEvent>>) {
            self.text = self.textField.getText();
        }
        operation changedUpdate(e:<<javax.swing.event.DocumentEvent>>) {
            self.text = self.textField.getText();
        }
    };
    text = value;
    textField.getDocument().addDocumentListener(documentListener);
    return textField;
}

trigger on (TextField.editable = value) {
    textField.setEditable(value);
}

trigger on (TextField.verify = newValue) {
    if (verifier == null) {
        verifier = new <<net.java.javafx.ui.XInputVerifierImpl>>();
        var self = this;
        verifier.addInputVerifier(new <<net.java.javafx.ui.XInputVerifier>>() {
                                      operation verify(comp) {
                                          return (self.verify)(self.textField.getText());
                                      }
                                  });
        textField.setInputVerifier(verifier);
    }
}

trigger on (new TextField) {
    editable = true;
    textField = new <<javax.swing.JFormattedTextField>>();
    textField.setFont(<<javax.swing.UIManager>>.getFont("TextField.font"));
    var tf = textField;
    var self = this;
    propertyChangeListener = new <<java.beans.PropertyChangeListener>>() {
            nodebug operation propertyChange(e:<<java.beans.PropertyChangeEvent>>) {
                if (e.getPropertyName().equals("value")) {
                      do later {
                              self.value = tf.getText();
                             (self.onChange)(tf.getText());
                       }
                }
            }
    };
    textField.addActionListener(new <<java.awt.event.ActionListener>>() {
                                    operation actionPerformed(e) {
                                        self.value = self.textField.getText();
                                       (self.onChange)(tf.getText());
                                        if (self.action <> null) {
                                            (self.action)();
                                        } else {
                                              var root = tf.getRootPane();      
                                              var but = root.getDefaultButton();
                                              if (but <> null) {
                                                  but.doClick(0);
                                              }
                                        }
                                    }       
                                });     
    textField.setDragEnabled(true);
    textField.addPropertyChangeListener(propertyChangeListener);
    if (true) {        
    context:<<net.java.javafx.ui.UIContext>>.addTransferHandler(textField,
                                                        :String,
                                                        new <<net.java.javafx.ui.ValueGetter>>() {
                                                            operation get() {
                                                                return self.value;
                                                            }
                                                        },
                                                        new <<net.java.javafx.ui.ValueSetter>>() {
                                                            operation set(value) {
                                                                self.value = "{value}";
                                                                
                                                            }
                                                        },
                                                        new <<net.java.javafx.ui.ValueAcceptor>>() {
                                                            operation accept(value) {
                                                                return true;
                                                            }
                                                        },
                                                        new <<net.java.javafx.ui.VisualRepresentation>>() {
                                                            operation getComponent(value) {
                                                                var label = TextField {
                                                                    value: self.value
                                                                    border: self.border
                                                                    foreground: self.foreground
                                                                    background: self.background
                                                                    columns: self.columns
                                                                    value: value
                                                                };
                                                                return label.getComponent();
                                                            }
                                                        });
}
}

trigger on (TextField.value = x) {
    if (x <> textField.getText()) {
        textField.setText(x);
    }
}

trigger on (TextField.columns = n) {
    textField.setColumns(n);
}

trigger on (TextField.onChange[old] = value) {
    if (old == null and value <> null) {
        textField.addPropertyChangeListener(propertyChangeListener);
    } else if (old <> null) {
        textField.removePropertyChangeListener(propertyChangeListener);
    }
}
