/*
 *  $Id$
 * 
 *  Copyright 2007 Sun Microsystems, Inc. All rights reserved.
 *  SUN PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

package javafx.ui;
import javafx.ui.Border;
import javafx.ui.TitledBorderPosition;
import javafx.ui.TitledBorderJustification;



/**
 * A class which implements an arbitrary border
 * with the addition of a String title in a
 * specified position and justification.
 * <p>
 * If the border, titleFont, or titleColor attribute values are not 
 * specified in the constuctor or by invoking the appropriate
 * set methods, the property values will be defined by the current
 * look and feel, using the following property names in the
 * Swing UI Defaults Table:
 * <ul>
 * <li>&quot;TitledBorder.border&quot;
 * <li>&quot;TitledBorder.font&quot;
 * <li>&quot;TitledBorder.titleColor&quot;
 * </ul>
 * <p>
 */

public class TitledBorder extends Border {
    public attribute border: Border?;
    public attribute title: String;
    public attribute titlePosition: TitledBorderPosition;
    public attribute titleJustification: TitledBorderJustification;
    public attribute titleFont: Font?;
    public attribute titleColor: Color?;
}

operation TitledBorder.getBorder() {
    return new <<javax.swing.border.TitledBorder>>(border.getBorder(),
						       title,
						       titleJustification.id,
						       titlePosition.id,
						       titleFont.getFont(),
						       titleColor.getColor());
}
