/*
 *  $Id$
 * 
 *  Copyright 2007 Sun Microsystems, Inc. All rights reserved.
 *  SUN PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

package javafx.ui;
import javafx.ui.Tree;

public class TreeCell {
    protected operation getTree():Tree;
    protected attribute tree: Tree;
    protected attribute parent: TreeCell;


    public attribute text: String;
    public attribute toolTipText: String?;
    public attribute cells: TreeCell*;
    public attribute value: *?;
    public attribute selected: Boolean;

}

operation TreeCell.getTree() {
    if (tree <> null) {
        return tree;
    }
    var p = parent;
    while (p <> null) {
        if (p.tree <> null) {
            tree = p.tree;
            break;
        }
        p = p.parent;
    }
    return tree;
}

trigger on (insert row into TreeCell.cells) {
    row.parent = this;
    row.tree = this.getTree();
    this.getTree().fireRowInserted(row, indexof row);
}

trigger on TreeCell.selected = value {
    if (tree <> null and not tree.inSelectionChange) {
        var p:TreeCell* = [];
        var r = this;
        while (r <> null) {
            insert r into p;
            r = r.parent;
        }
        var tp = new <<javax.swing.tree.TreePath>>(p);
        if (value) {
            tree.tree.addSelectionPath(tp);
        } else {
            tree.tree.removeSelectionPath(tp);
        }
    }
}

trigger on (TreeCell.cells[old] = row) {
    row.parent = this;
    row.tree = tree;
    var i = indexof row;
    this.getTree().fireTreeStructureChanged(row);
}

trigger on (TreeCell.text = newValue) {
    this.getTree().fireNodeValueChanged(this);
}

trigger on (delete row from TreeCell.cells) {
    this.getTree().fireRowDeleted(row, indexof row);
}
