/*
 *  $Id$
 * 
 *  Copyright 2007 Sun Microsystems, Inc. All rights reserved.
 *  SUN PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

package javafx.ui;
import java.awt.Font as AWTFont;
import java.awt.Color as AWTColor;
import javafx.ui.Color;
import javafx.ui.Font;
import javafx.ui.Cursor;
import java.awt.Dimension;
import java.awt.ComponentOrientation;
import javafx.ui.Border;
import javafx.ui.GroupElement;
import javafx.ui.MouseEvent;
import javafx.ui.Spring;
import javafx.ui.UIElement;
import javafx.ui.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.FocusListener;
import java.awt.event.KeyEvent as AWTKeyEvent;
import javax.swing.SwingUtilities;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.event.MouseWheelListener;
import java.awt.event.ComponentListener;
import javafx.ui.KeyStroke;
import javax.swing.KeyStroke as JKeyStroke;
import javax.swing.Action;
import javafx.ui.canvas.*;
/**
 * Base class for non-window javafx graphical components
 * Encapsulates javax.swing.JComponent
*/

public class KeyboardAction {
    public attribute keyStroke: KeyStroke;
    public attribute enabled: Boolean;
    public attribute action: function();
}

attribute KeyboardAction.enabled = true;

public class Widget extends GroupElement, UIElement {
    private attribute inBoundsListener: Boolean;
    private operation makeKeyEvent(e:AWTKeyEvent):KeyEvent;
    protected operation installMouseListener();
    protected operation installMouseWheelListener();
    protected operation installMouseMotionListener();
    private operation makeMouseEvent(e:<<java.awt.event.MouseEvent>>):MouseEvent;
    private operation makeMouseWheelEvent(e:<<java.awt.event.MouseWheelEvent>>):MouseEvent;
    protected attribute mouseListener: MouseListener?;
    protected attribute mouseWheelListener: MouseWheelListener?;
    private attribute focusListener: FocusListener?;
    private attribute keyListener: KeyListener?;
    protected attribute mouseMotionListener: MouseMotionListener?;
    protected operation onSetOpaque(value:Boolean);

    operation setBounds(b:<<java.awt.Rectangle>>);
    operation getBounds():<<java.awt.Rectangle>>;

    public attribute keyboardAction: KeyboardAction*;

    public attribute focusTraversalKeysEnabled: Boolean?;

    public operation requestFocus();

    /**
     *  Sets whether the this component should use a buffer to paint.
     *  If set to true, all the drawing from this component will be done
     *  in an offscreen painting buffer. The offscreen painting buffer will
     *  the be copied onto the screen.
     */
    public attribute doubleBuffered: Boolean?;

    /** Sets the x coordinate of this component within its parent. Has no effect unless contained in a Panel. */
    public attribute x: Number?;

    /** Sets the y coordinate of this component within its parent. Has no effect unless contained in a Panel. */
    public attribute y: Number?;

    /** Sets width of this component. Has no effect unless contained in a Panel.  */
    public attribute width: Number?;

    /** Sets the height of this component. Has no effect unless contained in a Panel.  */
    public attribute height: Number?;

    /** read-only attribute providing access to the underlying Swing component */
    public attribute component: <<javax.swing.JComponent>>;

    /** Hint to GroupLayout to make this component's height equal to others in the same row */
    public attribute sizeToFitRow: Boolean?;

    /** Hint to GroupLayout to make this component's width equal to others in the same column */
    public attribute sizeToFitColumn: Boolean?;

    /**
     * Registers the text to display in a tool tip.
     * The text displays when the cursor lingers over the component.
     */
    public attribute toolTipText: String?;

    /**
     * True when the object is visible. An object that is not
     * visible is not drawn on the screen.
     */
    public attribute visible: Boolean?;

    /**
     * Sets the background color of this component.
     */
    public attribute background: AbstractColor?;
    protected attribute awtBackground: AWTColor?;
    /**
     * Sets the foreground color of this component.
     */
    public attribute foreground: Color?;
    protected attribute awtForeground: AWTColor?;

    /**
     * If true the component paints every pixel within its bounds. 
     * Otherwise, the component may not paint some or all of its
     * pixels, allowing the underlying pixels to show through.
     * The default value of this attribute is <code>false</code>.
     */
    public attribute opaque: Boolean?;

    /**
     * Sets the <code>Font</code> of this object.
     */
    public attribute font: Font?;
    protected attribute awtFont: AWTFont?;
    /**
     * Sets the focusable state of this Component to the specified value. This
     * value overrides the Component's default focusability.
     */
    public attribute focusable: Boolean?;

    public attribute focused: Boolean;
    /**
     * Resizes this component to the specified dimensions.
     */
    public attribute size: Dimension?;
    /*
     * Sets the preferred size of this component.
     * If <code>preferredSize</code> is <code>null</code>, the UI will
     * be asked for the preferred size.
     */
    public attribute preferredSize: Dimension?;

    /**
     * Sets the maximum size of this component to a constant
     * value.  Subsequent calls to <code>maximumSize()</code> will always
     * return this value; the component's UI will not be asked
     * to compute it.  Setting the maximum size to <code>null</code>
     * restores the default behavior.
     */
    public attribute maximumSize: Dimension?;
    /**
     * Sets the minimum size of this component to a constant
     * value.  Subsequent calls to <code>minimumSize()</code> will always
     * return this value; the component's UI will not be asked
     * to compute it.  Setting the minimum size to <code>null</code>
     * restores the default behavior.
     */
    public attribute minimumSize: Dimension?;

    /**
     * Sets the border of this component.  The <code>Border</code> object is
     * responsible for defining the insets for the component
     * (overriding any insets set directly on the component) and
     * for optionally rendering any border decorations within the
     * bounds of those insets.  Borders should be used (rather
     * than insets) for creating both decorative and non-decorative
     * (such as margins and padding) regions for a swing component.
     * Compound borders can be used to nest multiple borders within a
     * single component.
     */
    public attribute border: Border?;

    /**
     * Sets the cursor image to the specified cursor.  This cursor
     * image is displayed when the <code>contains</code> method for
     * this component returns true for the current cursor location, and
     * this Component is visible, displayable, and enabled. Setting the
     * cursor of a <code>Container</code> causes that cursor to be displayed
     * within all of the container's subcomponents, except for those
     * that have a non-<code>null</code> cursor. 
     */
    public attribute cursor: Cursor?;
    /**
     * True when the object is enabled. An object that is not
     * enabled does not interact with the user. Defaults to <code>true</code>.
     */
    public attribute enabled: Boolean?;
    /**
     * Returns the alignment along the x axis.  This specifies how
     * the component would like to be aligned relative to other 
     * components.  The value should be a number between 0 and 1
     * where 0 represents alignment along the origin, 1 is aligned
     * the furthest away from the origin, 0.5 is centered, etc.
     */
    public attribute alignmentX: Number?;
    /**
     * Returns the alignment along the y axis.  This specifies how
     * the component would like to be aligned relative to other 
     * components.  The value should be a number between 0 and 1
     * where 0 represents alignment along the origin, 1 is aligned
     * the furthest away from the origin, 0.5 is centered, etc.
     */
    public attribute alignmentY: Number?;

    /**
     * Sets the language-sensitive orientation that is to be used to order
     * the elements or text within this component.  Language-sensitive
     * <code>LayoutManager</code> and <code>Component</code>
     * subclasses will use this property to
     * determine how to lay out and draw components.
     */
    public attribute componentOrientation: ComponentOrientation?;
    /** factory method to create the underlying Swing component*/
    protected operation createComponent():<<javax.swing.JComponent>>;
    public operation getComponent():<<javax.swing.JComponent>>;
    public operation getNonScrollPaneComponent():<<javax.swing.JComponent>>;
    /** Returns the current preferred size of this component */
    public operation preferredSize(): Dimension;
    /** Returns the current minimum size of this component */
    public operation minimumSize(): Dimension;
    /** Returns the current maximum size of this component */
    public operation maximumSize(): Dimension;

    public operation size(): Dimension;

    /**
     * <code>attribute onMouseEntered: function(e:MouseEvent)</code><br></br>
     * Optional handler for mouse enter events.
     */
    public attribute onMouseEntered: function(e:MouseEvent);
    /**
     * <code>attribute onMouseExited: function(e:MouseEvent)</code><br></br>
     * Optional handler for mouse exit events.
     */
    public attribute onMouseExited: function(e:MouseEvent);
    /**
     * <code>attribute onMouseMoved: function(e:MouseEvent)</code><br></br>
     * Optional handler for mouse motion events.
     */
    public attribute onMouseMoved: function(e:MouseEvent);
    /**
     * <code>attribute onMousePressed: function(e:MouseEvent)</code><br></br>
     * Optional handler for mouse press events.
     */
    public attribute onMousePressed: function(e:MouseEvent);
    /**
     * <code>attribute onMouseReleased: function(e:MouseEvent)</code><br></br>
     * Optional handler for mouse release events.
     */
    public attribute onMouseReleased: function(e:MouseEvent);
    /**
     * <code>attribute onMouseClicked: function(e:MouseEvent)</code><br></br>
     * Optional handler for mouse exit events.
     */
    public attribute onMouseClicked: function(e:MouseEvent);

    public attribute onMouseDragged: function(e:MouseEvent);

    public attribute onMouseWheelMoved: function(e:MouseWheelEvent);

    /**
     * <code>attribute onKeyUp: function(event:KeyEvent);
     * Optional handler for key release events.
     */
    public attribute onKeyUp: function(event:KeyEvent);
    /**
     * <code>attribute onKeyDown: function(event:KeyEvent);
     * Optional handler for key press events.
     */
    public attribute onKeyDown: function(event:KeyEvent);

    /**
     * <code>attribute onKeyTyped: function(event:KeyEvent)
     * Optional handler for key typed events.
     */
    public attribute onKeyTyped: function(event:KeyEvent);
}
attribute Widget.awtFont = bind font.getFont();
attribute Widget.awtBackground = bind background.getColor();
attribute Widget.awtForeground = bind foreground.getColor();


trigger on Widget.cursor = value {
    component.setCursor(value.getCursor());
}

trigger on Widget.doubleBuffered = value {
    component.setDoubleBuffered(value);
}

operation Widget.preferredSize():Dimension {
    return component.getPreferredSize();
}

operation Widget.requestFocus() {
    this.getNonScrollPaneComponent().requestFocus();
}

operation Widget.minimumSize():Dimension {
    return component.getMinimumSize();
}

operation Widget.maximumSize():Dimension {
    return component.getMaximumSize();
}

operation Widget.size():Dimension {
    return component.getSize();
}

trigger on (Widget.preferredSize = value) {
    if (component <> null) {
        if (value.height <= 0) {
            value.height = component.getPreferredSize().height;
        }
        if (value.width <= 0) {
            value.width = component.getPreferredSize().width;
        }
        component.setPreferredSize(value);
    }
}

trigger on (Widget.maximumSize = value) {
    if (component <> null) {
        if (value.height <= 0) {
            value.height = component.getMaximumSize().height;
        }
        if (value.width <= 0) {
            value.width = component.getMaximumSize().width;
        }
        component.setMaximumSize(value);
    }
}

trigger on (Widget.minimumSize = value) {
    if (component <> null) {
        if (value.height <= 0) {
            value.height = component.getMinimumSize().height;
        }
        if (value.width <= 0) {
            value.width = component.getMinimumSize().width;
        }
        component.setMinimumSize(value);
    }
}

operation Widget.getNonScrollPaneComponent() {
    var w = this.getComponent();
    if (w instanceof <<javax.swing.JScrollPane>>) {
        w = (<<javax.swing.JComponent>>)((<<javax.swing.JScrollPane>>)w).getViewport().getView();
    }
    return w;
}

operation Widget.getWindow() {
    return <<javax.swing.SwingUtilities>>.getWindowAncestor(this.getComponent());
}


operation Widget.getComponent():<<javax.swing.JComponent>> {
    if (component == null) {
        var comp = this.createComponent();
        var c = comp;
        if (c instanceof <<javax.swing.JScrollPane>>) {
            c = (<<javax.swing.JComponent>>)((<<javax.swing.JScrollPane>>)c).getViewport().getView();
        }
        if (doubleBuffered <> null) {
            c.setDoubleBuffered(doubleBuffered);
        }
        if (cursor <> null) {
            c.setCursor(cursor.getCursor());
        }
        if (toolTipText <> null) {
            c.setToolTipText(toolTipText);
        }
        if (visible <> null) {
            c.setVisible(visible);
        }
        if (opaque <> null) {
            c.setOpaque(opaque);
        }
        if (background <> null) {
            c.setBackground(background.getColor());
            c.setOpaque(true);
        }
        if (foreground <> null) {
            c.setForeground(foreground.getColor());
        }
        if (awtFont <> null) {
            c.setFont(awtFont);
        }
        if (focusable <> null) {
            if (focusable <> c.isFocusable()) {
               c.setFocusable(focusable);
            }
        }
        var self = this;
        focusListener = new FocusListener() {
                operation focusGained(e) {
                    self.focused = true;
                }
                operation focusLost(e) {
                    self.focused = false;
                }
           };
        c.addFocusListener(focusListener);
        if (componentOrientation <> null) {
            c.setComponentOrientation(componentOrientation);
        }
        if (border <> null) {
            try {
               c.setBorder(border.getBorder());
            } catch (e) {
            }
        }
        if (enabled <> null) {
            c.setEnabled(enabled);
        }
        if (size <> null) {
            c.setSize(size);
        }
        if (alignmentX <> null) {
            c.setAlignmentX(alignmentX);
        }
        if (preferredSize <> null) {
            var dim = c.getPreferredSize();
            if (dim <> null) {
                if (preferredSize.height <= 0) {
                    preferredSize.height = dim.height;
                }
                if (preferredSize.width <= 0) {
                    preferredSize.width = dim.width;
                }
            }
            c.setPreferredSize(preferredSize);
        }
        if (alignmentY <> null) {
            c.setAlignmentY(alignmentY);
        }
        if (keyboardAction <> null) {
            var inputMap = c.getInputMap();
            var actionMap = c.getActionMap();
            for (i in keyboardAction) {
                var k = i.keyStroke;
                var a = i.action;
                var j = JKeyStroke.getKeyStroke(k.id, 0/*fix me*/);
                inputMap.put(j, i);
                actionMap.put(i, new Action() {
                        operation isEnabled() {
                            return i.enabled;
                        }
                        operation actionPerformed(e) {
                            (i.action)();
                        }
                    });
            }
        }
        if (focusTraversalKeysEnabled <> null) {
            c.setFocusTraversalKeysEnabled(focusTraversalKeysEnabled);
        }
        if (keyListener <> null) {
            c.addKeyListener(keyListener);
        }
        if (mouseListener <> null) {
            c.addMouseListener(mouseListener);
        }
        if (mouseMotionListener <> null) {
            c.addMouseMotionListener(mouseMotionListener);
        }
        if (mouseWheelListener <> null) {
            c.addMouseWheelListener(mouseWheelListener);
        }
        self.inBoundsListener = true;
        width = comp.getWidth();
        height = comp.getHeight();
        self.inBoundsListener = false;
        comp.addComponentListener(new ComponentListener() {
              operation componentResized(e) {
                  self.inBoundsListener = true;
                  self.height = comp.getHeight();
                  self.width = comp.getWidth();
                  self.inBoundsListener = false;
              }
        });
        component = comp;
    }
    return component;
}

trigger on Widget.onKeyUp = value {
    if (value <> null) {
        installKeyListener();
    }
}

trigger on Widget.onKeyDown = value {
    if (value <> null) {
        installKeyListener();
    }
}

trigger on Widget.onKeyTyped = value {
    if (value <> null) {
        installKeyListener();
    }
}

operation Widget.installKeyListener() {
    if (keyListener == null) {
        var self = this;
        keyListener = new KeyListener() {
                operation keyTyped(e:AWTKeyEvent) {
                    (self.onKeyTyped)(self.makeKeyEvent(e));
                }
                operation keyPressed(e:AWTKeyEvent) {
                    (self.onKeyDown)(self.makeKeyEvent(e));
                }
                operation keyReleased(e:AWTKeyEvent) {
                    (self.onKeyUp)(self.makeKeyEvent(e));
                }
            };
        if (component <> null) {
            getNonScrollPaneComponent().addKeyListener(keyListener);
        }
    }
}

operation Widget.makeMouseEvent(e:<<java.awt.event.MouseEvent>>) {
    return MouseEvent {
        modifiers: [if e.isAltDown() then ALT else null,
                    if e.isControlDown() then CTRL else null, 
                    if e.isMetaDown() then META else null,
                    if e.isShiftDown() then SHIFT else null]
        clickCount: e.getClickCount()
        button: if SwingUtilities.isLeftMouseButton(e) then 1 else if SwingUtilities.isRightMouseButton(e) then 3 else 2
        x: e.getX()
        y: e.getY()
        source: e
    };
}

operation Widget.makeMouseWheelEvent(e:<<java.awt.event.MouseWheelEvent>>) {
    return MouseWheelEvent {
        modifiers: [if e.isAltDown() then ALT else null,
                    if e.isControlDown() then CTRL else null, 
                    if e.isMetaDown() then META else null,
                    if e.isShiftDown() then SHIFT else null]
        clickCount: e.getClickCount()
        button: if SwingUtilities.isLeftMouseButton(e) then 1 else if SwingUtilities.isRightMouseButton(e) then 3 else 2
        x: e.getX()
        y: e.getY()
        scrollType: if e.getScrollType() == e.WHEEL_UNIT_SCROLL then UNIT_SCROLL else BLOCK_SCROLL
        scrollAmount: e.getScrollAmount()
        wheelRotation: e.getWheelRotation()
        unitsToScroll: e.getUnitsToScroll()
        source: e
    };
}

operation Widget.installMouseMotionListener() {
    if (mouseMotionListener == null) {
        var self = this;
        mouseMotionListener = new MouseMotionListener() {
                operation mouseMoved(e:<<java.awt.event.MouseEvent>>) {
                    (self.onMouseMoved)(self.makeMouseEvent(e));
                }
                operation mouseDragged(e:<<java.awt.event.MouseEvent>>) {
                    (self.onMouseDragged)(self.makeMouseEvent(e));
                }
            };
        if (component <> null) {
            this.getNonScrollPaneComponent().addMouseMotionListener(mouseMotionListener);
        }
    }
}

operation Widget.installMouseListener() {
    if (mouseListener == null) {
        var self = this;
        mouseListener = new MouseListener() {
                operation mouseEntered(e:<<java.awt.event.MouseEvent>>) {
                    (self.onMouseEntered)(self.makeMouseEvent(e));
                }
                operation mouseExited(e:<<java.awt.event.MouseEvent>>) {
                    (self.onMouseExited)(self.makeMouseEvent(e));
                }
                operation mouseClicked(e:<<java.awt.event.MouseEvent>>) {
                    (self.onMouseClicked)(self.makeMouseEvent(e));
                }
                operation mouseReleased(e:<<java.awt.event.MouseEvent>>) {
                    (self.onMouseReleased)(self.makeMouseEvent(e));
                }
                operation mousePressed(e:<<java.awt.event.MouseEvent>>) {
                    (self.onMousePressed)(self.makeMouseEvent(e));
                }
            };
        if (component <> null) {
            this.getNonScrollPaneComponent().addMouseListener(mouseListener);
        }
    }
}

operation Widget.installMouseWheelListener() {
    if (mouseWheelListener == null) {
        var self = this;
        mouseWheelListener = new MouseWheelListener() {
                operation mouseWheelMoved(e:<<java.awt.event.MouseWheelEvent>>) {
                    (self.onMouseWheelMoved)(self.makeMouseWheelEvent(e));
                }
            };
        if (component <> null) {
            this.getNonScrollPaneComponent().addMouseWheelListener(mouseWheelListener);
        }
    }
}

trigger on (Widget.componentOrientation = value) {
    if (component <> null) {
        this.getNonScrollPaneComponent().setComponentOrientation(value);
    }
}

trigger on (Widget.size = value) {
    component.setSize(value);
}

trigger on Widget.toolTipText = value {
    if (component <> null) {
        this.getNonScrollPaneComponent().setToolTipText(value);
    }
}

trigger on (Widget.alignmentX = value) {
    if (value <> null) {
        component.setAlignmentX(value);
    }
}

trigger on (Widget.alignmentY = value) {
    if (value <> null) {
        component.setAlignmentY(value);
    }
}

trigger on (Widget.border = value) {
    var c:<<java.awt.Component>> = component;
    if (c <> null) {
        if (c instanceof <<javax.swing.JScrollPane>>) {
            c = ((<<javax.swing.JScrollPane>>)component).getViewport().getView();
        } 
        if (c instanceof <<javax.swing.JComponent>>) {
            try {
               ((<<javax.swing.JComponent>>)c).setBorder(value.getBorder());
            } catch (e) {
            }
        }
    }
}

trigger on (Widget.focusable = value) {
    if (value <> null) {
        if (component <> null) {
            if (component instanceof <<javax.swing.JScrollPane>>) {
                ((<<javax.swing.JScrollPane>>)component).getViewport().getView().setFocusable(value);
            } else {
                component.setFocusable(value);
            }
        }
    }
}

trigger on (Widget.visible = value) {
    if (component <> null) {
        component.setVisible(value);
    }
}

trigger on (Widget.opaque = value) {
    this.onSetOpaque(value);
}


operation Widget.onSetOpaque(value:Boolean) {
    if (component <> null) {
        component.setOpaque(value);
    }
}

trigger on (Widget.awtBackground = color) {
    if (component <> null) {
        var c = this.getNonScrollPaneComponent();
        if (color <> null) {
            opaque = true;
        }
        c.setBackground(color);
    }
}

trigger on (Widget.awtForeground = color) {
    if (component <> null) {
        var c = this.getNonScrollPaneComponent();
        if (color <> null) {
            c.setForeground(color);
        }
    }
}

trigger on (Widget.enabled = value) {
    if (component <> null) {
        var c = this.getNonScrollPaneComponent();
        c.setEnabled(value);
    }
}

trigger on (Widget.awtFont = f) {
    if (component <> null) {
        var c = this.getNonScrollPaneComponent();
        c.setFont(f);
    }
}

trigger on Widget.x = n {
    if (inBoundsListener) {return;}
    var b = this.getBounds();
    b.x = n;
    this.setBounds(b);
}

trigger on Widget.y = n {
    if (inBoundsListener) {return;}
    var b = this.getBounds();
    b.y = n;
    this.setBounds(b);
}

trigger on Widget.width = n {
    if (inBoundsListener) {return;}
    println("width {getComponent()} = {n}");
    var b = this.getBounds();
    b.width = n;
    this.setBounds(b);
}

trigger on Widget.height = n {
    if (inBoundsListener) {return;}
    println("height {getComponent()} = {n}");
    var b = this.getBounds();
    b.height = n;
    this.setBounds(b);
}

operation Widget.setBounds(b:<<java.awt.Rectangle>>) {
    component.setBounds(b);     
}

operation Widget.getBounds():<<java.awt.Rectangle>> {
    return component.getBounds();     
}

trigger on Widget.onMouseClicked = value {
    installMouseListener();
}

trigger on Widget.onMouseWheelMoved = value {
    installMouseWheelListener();
}

trigger on Widget.onMousePressed = value {
    installMouseListener();
}

trigger on Widget.onMouseReleased = value {
    installMouseListener();
}

trigger on Widget.onMouseEntered = value {
    installMouseMotionListener();
}

trigger on Widget.onMouseExited = value {
    installMouseListener();
}

trigger on Widget.onMouseMoved = value {
    installMouseListener();
}

operation Widget.makeKeyEvent(e:AWTKeyEvent) {
    var modifiers:KeyStroke* = [if e.isControlDown() then CONTROL:KeyStroke else [], if e.isShiftDown() then SHIFT:KeyStroke else [], if e.isMetaDown() then META:KeyStroke else if e.isAltDown() then ALT:KeyStroke else []];
    return KeyEvent {
        source: e
        keyStroke: KEYBOARD:Keyboard.getKeyStroke(e.getKeyCode())
        modifiers: modifiers
        keyChar: if e.getID() == AWTKeyEvent.KEY_TYPED then "{e.getKeyChar()}" else null
    };
}

trigger on insert i into Widget.keyboardAction {
    var c = this.getNonScrollPaneComponent();
    var inputMap = c.getInputMap();
    var actionMap = c.getActionMap();
    var k = i.keyStroke;
    var a = i.action;
    var j = JKeyStroke.getKeyStroke(k.id, 0/*fix me*/);
    inputMap.put(j, i);
    actionMap.put(i, new Action() {
            operation isEnabled() {
                return i.enabled;
            }
            operation actionPerformed(e) {
                (i.action)();
            }
        });
}

trigger on delete i from Widget.keyboardAction {
    var c = this.getNonScrollPaneComponent();
    var inputMap = c.getInputMap();
    var actionMap = c.getActionMap();
    var k = i.keyStroke;
    var j = JKeyStroke.getKeyStroke(k.id, 0/*fix me*/);
    inputMap.remove(j);
    actionMap.remove(i);

}

trigger on Widget.keyboardAction[oldValue] = newValue {
    var c = this.getNonScrollPaneComponent();
    var inputMap = c.getInputMap();
    var actionMap = c.getActionMap();
    var k = newValue.keyStroke;
    var j = JKeyStroke.getKeyStroke(k.id, 0/*fix me*/);
    inputMap.put(j, newValue);
    actionMap.remove(oldValue);
    actionMap.put(newValue, new Action() {
            operation isEnabled() {
                return newValue.enabled;
            }
            operation actionPerformed(e) {
                (newValue.action)();
            }
        });
}


trigger on Widget.focusTraversalKeysEnabled = value {
    if (component <> null) {
        this.getNonScrollPaneComponent().setFocusTraversalKeysEnabled(value);
    }
}
