/*
 *  $Id$
 * 
 *  Copyright 2007 Sun Microsystems, Inc. All rights reserved.
 *  SUN PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

package javafx.ui.canvas;
import javafx.ui.canvas.Shape;
import java.awt.geom.Area as JArea;
import java.awt.geom.Rectangle2D$Double as Rectangle2D;
import net.java.javafx.jazz.component.ZShape;

/**
 * The <code>Area</code> class is a device-independent specification of an
 * arbitrarily-shaped area.  The <code>Area</code> object is defined as an
 * object that performs certain binary CAG (Constructive Area Geometry)
 * operations on other area-enclosing geometries, such as rectangles,
 * ellipses, and polygons. The CAG operations are Add(union), Subtract,
 * Intersect, and ExclusiveOR. For example, an <code>Area</code> can be
 * made up of the area of a rectangle minus the area of an ellipse.
 *
 * This is an abstract class. The specific operations are provided by
 * subclasses of Area: <code>Add</code>, <code>Subtract</code>, 
 * <code>Intersect</code>, and <code>XOR</code>.
 */
public class Area extends Shape {
    protected operation performOp(area1:JArea, area2:JArea);
    private operation getArea();
    /** the shape which is the first operand to this area operation */
    public attribute shape1: Shape?;
    /** the shape which is the second operand to this area operation */
    public attribute shape2: Shape?;
    ...
}

operation Area.createShape() {
    zshape = new ZShape();
    this.getArea();
    return zshape;
}

operation Area.getArea() {
    if (zshape <> null) {
        var s1 = shape1.getTransformedShape();
        var s2 = shape2.getTransformedShape();
        if (s1 == null) {
            s1 = new Rectangle2D();
        }
        if (s2 == null) {
            s2 = new Rectangle2D();
        }
	var area1 = new JArea(s1);
	var area2 = new JArea(s2);
	this.performOp(area1, area2);
	zshape.setShape(area1);
    }
}

trigger on Area.shape1 = shape {
    this.getArea();
}

trigger on Area.shape2 = shape {
    this.getArea();
}
