/*
 *  $Id$
 * 
 *  Copyright 2007 Sun Microsystems, Inc. All rights reserved.
 *  SUN PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

package javafx.ui.canvas;
import javafx.ui.canvas.Shape;
import net.java.javafx.jazz.component.ZCubicCurve;


/**
 * The <code>CubicCurve</code> class defines a cubic parametric curve 
 * segment by means of a start point (x1, y1), two control points, 
 * (ctrlx1, ctrly1), (ctrlx2, ctrly2), and an end point (x2, y2).
 */

public class CubicCurve extends Shape {
    //private
    private attribute zcubiccurve: ZCubicCurve;

    /** The  x coordinate of the start point of this cubic curve. */
    public attribute x1: Number;
    /** The  y coordinate of the start point of this cubic curve. */
    public attribute y1: Number;
    
    /** The x coordinate of the first control point of this cubic curve. */
    public attribute ctrlx1: Number;
    /** The y coordinate of the first control point of this cubic curve. */
    public attribute ctrly1: Number;

    /** The x coordinate of the second control point of this cubic curve. */
    public attribute ctrlx2: Number;
    /** The y coordinate of the second control point of this cubic curve. */
    public attribute ctrly2: Number;

    /** The x coordinate of the endpoint of this cubic curve */
    public attribute x2: Number;
    /** The y coordinate of the endpoint of this cubic curve */
    public attribute y2: Number;
}

operation CubicCurve.createShape() {
    zcubiccurve = new ZCubicCurve(x1, y1, ctrlx1, ctrly1, ctrlx2, ctrly2, x2, y2);
    return zcubiccurve;
}

trigger on CubicCurve.x1 = newValue {
    if (zcubiccurve <> null) {
	zcubiccurve.setCurve(x1, y1, ctrlx1, ctrly1, ctrlx2, ctrly2, x2, y2);
    }
}

trigger on CubicCurve.y1 = newValue {
    if (zcubiccurve <> null) {
	zcubiccurve.setCurve(x1, y1, ctrlx1, ctrly1, ctrlx2, ctrly2, x2, y2);
    }
}

trigger on CubicCurve.ctrlx1 = newValue {
    if (zcubiccurve <> null) {
	zcubiccurve.setCurve(x1, y1, ctrlx1, ctrly1, ctrlx2, ctrly2, x2, y2);
    }
}

trigger on CubicCurve.ctrly1 = newValue {
    if (zcubiccurve <> null) {
	zcubiccurve.setCurve(x1, y1, ctrlx1, ctrly1, ctrlx2, ctrly2, x2, y2);
    }
}

trigger on CubicCurve.ctrlx2 = newValue {
    if (zcubiccurve <> null) {
	zcubiccurve.setCurve(x1, y1, ctrlx1, ctrly1, ctrlx2, ctrly2, x2, y2);
    }
}

trigger on CubicCurve.ctrly2 = newValue {
    if (zcubiccurve <> null) {
	zcubiccurve.setCurve(x1, y1, ctrlx1, ctrly1, ctrlx2, ctrly2, x2, y2);
    }
}

trigger on CubicCurve.x2 = newValue {
    if (zcubiccurve <> null) {
	zcubiccurve.setCurve(x1, y1, ctrlx1, ctrly1, ctrlx2, ctrly2, x2, y2);
    }
}

trigger on CubicCurve.y2 = newValue {
    if (zcubiccurve <> null) {
	zcubiccurve.setCurve(x1, y1, ctrlx1, ctrly1, ctrlx2, ctrly2, x2, y2);
    }
}
