/*
 *  $Id$
 * 
 *  Copyright 2007 Sun Microsystems, Inc. All rights reserved.
 *  SUN PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

package javafx.ui.canvas;
import net.java.javafx.jazz.event.ZNodeListener;
import net.java.javafx.jazz.event.ZNodeEvent;
import net.java.javafx.jazz.ZCamera;
import net.java.javafx.jazz.util.ZBounds;
import net.java.javafx.jazz.ZLayerGroup;
import net.java.javafx.jazz.ZVisualLeaf;
import java.awt.Composite;
import java.awt.geom.AffineTransform;
import javafx.ui.Canvas;
import javafx.ui.Paint;
/**
 * Provides a view onto some other part of the scene graph. The node 
 * assigned to the <code>view</code> attribute will also be rendered
 * within the bounds of this object and this object's transformations
 * and/or filters, opacity, etc will be applied to it.
 */

public class Lens extends Node {
    private attribute lgx: Number;
    private attribute lgy: Number;
    private attribute nodeListener:ZNodeListener;
    private attribute zcamera: ZCamera;
    private attribute lg: ZLayerGroup;
    private attribute viewBounds: ZBounds?;
    private operation setView();
    public attribute composite: Composite?;
    public attribute viewComposite: Composite?;
    public attribute fill: Paint?;
    public attribute mask: Paint?;
    // public
    /** the object that will be displayed in this lens */
    public attribute view: Node?;
}


attribute Lens.viewBounds = bind view.bounds;

trigger on Lens.composite = value {
    zcamera.setComposite(value);
}

trigger on Lens.viewComposite = value {
    zcamera.setComposite(value);
}

trigger on Lens.viewBounds = value {
    if (value <> null) {
        this.setView();
    }
}

trigger on Lens.view = value {
    value.parentCanvasElement = this;
    if (zcamera <> null) {
        if (lg <> null) {
            if (value.getNode() <> null) {
                if (lg.getNumChildren() > 0) {
                    lg.removeChild(0);
                }
                lg.addChild(value.getNode());
                zcamera.globalBoundsChanged(lg, null, lg.getBounds());
            }
        } else {
            lg = new ZLayerGroup(value.getNode());
            var self = this;
            lg.addNodeListener(new ZNodeListener() {
                    operation boundsChanged(e:ZNodeEvent) {
                        self.zcamera.globalBoundsChanged(self.lg, null, self.lg.getBounds());
                    }
                });
            zcamera.addLayer(lg);
        }
    }
}

operation Lens.createNode() {
    var self = this;
    var zcanvas = this.getCanvas().zcanvas;
    zcamera = new ZCamera();
    zcamera.setTracksLayerLocation(true);        
    var n = view.getNode();
    if (n <> null) {
        lg = new ZLayerGroup(n);
        lg.addNodeListener(new ZNodeListener() {
                operation boundsChanged(e:ZNodeEvent) {
                    self.zcamera.globalBoundsChanged(self.lg, null, self.lg.getBounds());
                }
            });
        zcamera.addLayer(lg);
    }
    zcamera.setDrawingSurface(zcanvas.getDrawingSurface());
    if (fill <> null) {
        zcamera.setFillPaint(fill.getPaint());
    }
    if (mask <> null) {
        zcamera.setMaskPaint(mask.getPaint());
    }
    if (composite <> null) {
        zcamera.setComposite(composite);
    }
    if (viewComposite <> null) {
        zcamera.setViewComposite(viewComposite);
    }
    return new ZVisualLeaf(zcamera);
}
