/*
 *  $Id$
 * 
 *  Copyright 2007 Sun Microsystems, Inc. All rights reserved.
 *  SUN PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

package javafx.ui.canvas;
import net.java.javafx.jazz.ZNode;
import net.java.javafx.jazz.event.ZMouseEvent;
import net.java.javafx.jazz.event.ZMouseListener;
import net.java.javafx.jazz.event.ZMouseMotionListener;
import net.java.javafx.jazz.event.ZNodeListener;
import net.java.javafx.jazz.event.ZNodeEvent;
import net.java.javafx.jazz.util.ZBounds;
import net.java.javafx.jazz.ZHandleGroup;
import net.java.javafx.jazz.ZLayerGroup;
import net.java.javafx.jazz.ZClipGroup;
import net.java.javafx.jazz.ZFilterGroup;
import net.java.javafx.jazz.ZGroup;
import net.java.javafx.jazz.ZFadeGroup;
import net.java.javafx.jazz.ZVisualComponent;
import net.java.javafx.jazz.ZSceneGraphObject;
import net.java.javafx.jazz.ZTransformGroup;
import net.java.javafx.jazz.ZAlignmentGroup;
import java.lang.Math;
import javafx.ui.Cursor;
import java.awt.geom.AffineTransform;
import java.awt.geom.Area as JArea;
import javax.swing.SwingUtilities;
import javafx.ui.Canvas;
import javafx.ui.canvas.CanvasElement;
import javafx.ui.canvas.CanvasMouseEvent;
import javafx.ui.HorizontalAlignment;
import javafx.ui.VerticalAlignment;
import javafx.ui.KeyEvent;
import javafx.ui.XY;
import java.awt.Point;
import javafx.ui.filter.Filter;
import java.awt.geom.Rectangle2D$Double as JRectangle2D;
import java.lang.System;
import net.java.javafx.ui.F3NodeListener;

MOUSE_PRESS:ZMouseEvent = null;
MOUSE_DRAG:ZMouseEvent = null;
MOUSE_DRAG_SCREEN:Point = null;

/**
 * Common base class for all objects that appear in a Canvas.
 */

public class Node extends CanvasElement, Transformable {
    // private:
    private attribute cachedCanvas: Canvas?;
    protected attribute alignmentTransform: AffineTransform?;
    protected attribute mouseMotionListener: ZMouseMotionListener;
    protected attribute mouseListener: ZMouseListener;
    protected operation installMouseListener();
    protected operation installMouseMotionListener();
    protected operation makeCanvasMouseEvent(e:ZMouseEvent);
    protected attribute tg: ZTransformGroup;
    protected attribute ag: ZAlignmentGroup;
    protected attribute fadeGroup: ZFadeGroup;
    private attribute cg: ZClipGroup;
    private attribute clipNode: Node?;
    private attribute antialiasClip: Boolean?;
    protected attribute filterGroup: ZFilterGroup;
    protected attribute sg: ZNode;
    //protected attribute _bounds: ZBounds?;
    protected attribute bounds: ZBounds?;
 
    private operation getF3(obj:ZSceneGraphObject): Node;



    // protected:
    protected operation getNode(): ZNode;
    protected operation createNode(): ZNode;
    protected operation getVisualComponent(): ZVisualComponent;
    
    public attribute clip: Clip?;
    public attribute toolTipText: String?;

    // public:
    /** Determines whether this node responds to mouse events, or other picking operations. */
    public attribute selectable: Boolean;
    public attribute isSelectionRoot: Boolean;

    public attribute exportDrag: Boolean;
    public attribute exportAsDrag: operation(): CanvasDragEvent;
    /**
     * An optional list of Filters that will be applied to this node. 
     * If present whenever the content of this node changes a new buffered 
     * image will be created consisting of the result of applying the list of 
     * filters (in order) to the new content. The generated image will then 
     * be used to paint the node.
     */
    public attribute filter: Filter*;
    /** A number between 0 and 1, 0 being transparent and 1 opaque. */
    public attribute opacity: Number?;

    /** If true this node will be scaled to the size of its containing canvas. */
    public attribute scaleToFitCanvas: Boolean;
    /** Determines the horizontal alignment of this node relative to its origin. */
    attribute halign: HorizontalAlignment?;
    /** Determines the vertical alignment of this node relative to its origin. */
    public attribute valign: VerticalAlignment?;

    /** 
     * Determines whether this node is visible in the canvas. If set to 
     * false, this node and anything it contains will not be painted and
     * will not receive events.
     */
    public attribute visible: Boolean;

    public attribute onKeyDown: operation(e:KeyEvent)?;
    public attribute onKeyUp: operation(e:KeyEvent)?;
    public attribute onKeyTyped: operation(e:KeyEvent)?;

    public attribute focusable: Boolean;

    public attribute focused: Boolean;

    attribute dragCount: Number;

    public attribute onDrop: function(e:CanvasDropEvent)?;
    public attribute canAcceptDrop: function(e:CanvasDropEvent):Boolean?;
    public attribute onDragEnter: function(e:CanvasDropEvent)?;
    public attribute onDragExit: function(e:CanvasDropEvent)?;

    operation handleDrop(e:CanvasDropEvent);
    operation handleAcceptDrop(e:CanvasDropEvent):Boolean;
    operation handleDragEnter(e:CanvasDropEvent):Boolean;
    operation handleDragExit(e:CanvasDropEvent):Boolean;
    operation doDragExport();
    
    /** 
     * <code>attribute onMouseClicked: function(e:CanvasMouseEvent)?</code><br></br>
     * Optional handler for mouse click events 
     */
    public attribute onMouseClicked: function(e:CanvasMouseEvent)?;
    /** 
     * <code>attribute onMouseEntered: function(e:CanvasMouseEvent)?</code><br></br>
     * Optional handler for mouse enter events. 
     */
    public attribute onMouseEntered: function(e:CanvasMouseEvent)?;
    /** 
     * <code>attribute onMouseExited: function(e:CanvasMouseEvent)?</code><br></br>
     * Optional handler for mouse exit events. 
     */
    public attribute onMouseExited: function(e:CanvasMouseEvent)?;
    /** 
     * <code>attribute onMousePressed: function(e:CanvasMouseEvent)?</code><br></br>
     * Optional handler for mouse press events. 
     */
    public attribute onMousePressed: function(e:CanvasMouseEvent)?;
    /** 
     * <code>attribute onMouseReleased: function(e:CanvasMouseEvent)?</code><br></br>
     * Optional handler for mouse release events. 
     */
    public attribute onMouseReleased: function(e:CanvasMouseEvent)?;
    /** 
     * <code>attribute onMouseDragged: function(e:CanvasMouseEvent)?</code><br></br>
     * Optional handler for mouse drag events. 
     */
    public attribute onMouseDragged: function(e:CanvasMouseEvent)?;
    /** 
     * <code>attribute onMouseMoved: function(e:CanvasMouseEvent)?</code><br></br>
     * Optional handler for mouse motion events. 
     */
    public attribute onMouseMoved: function(e:CanvasMouseEvent)?;
    /** Read-only attribute returning the current x coordinate of this node relative to its parent. */
    public attribute currentX: Number;
    /** Read-only attribute returning the current y coordinate of this node relative to its parent. */
    public attribute currentY: Number;
    /** Read-only attribute returning the current width of this node. */
    public attribute currentWidth: Number;
    /** Read-only attribute returning the current height of this node. */
    public attribute currentHeight: Number;
    /** Optional cursor to use when the mouse is over this node. */
    public attribute cursor: Cursor?;
    public operation getGlobalBounds(): JRectangle2D;
    public attribute hover: Boolean;
    public attribute id: String?;
    ...
}

attribute Node.antialiasClip = bind clip.antialias;

attribute Node.clipNode = bind clip.shape;

trigger on Node.antialiasClip = value {
    cg.setSmooth(value);
}

trigger on Node.clipNode = value {
    cg.setClipNode(value.getNode());
}

trigger on Node.id = value {
   getNode().putClientProperty("id", value);
   sg.putClientProperty("id", value);
}

operation Node.handleDrop(e:CanvasDropEvent) {
    if (onDrop <> null) {
        var pt = new <<java.awt.Point>>(e.x, e.y);
        ag.globalToLocal(pt);
        e.localX = pt.x;
        e.localY = pt.y;
        (this.onDrop)(e);
        return true;
    }
    return false;
}

operation Node.handleAcceptDrop(e:CanvasDropEvent) {
    if (canAcceptDrop <> null) {
        var pt = new <<java.awt.Point>>(e.x, e.y);
        ag.globalToLocal(pt);
        e.localX = pt.x;
        e.localY = pt.y;
        return (this.canAcceptDrop)(e);
    }
    return onDrop <> null;
}

operation Node.handleDragEnter(e:CanvasDropEvent) {
    if (onDragEnter <> null) {
        var pt = new <<java.awt.Point>>(e.x, e.y);
        ag.globalToLocal(pt);
        e.localX = pt.x;
        e.localY = pt.y;
        (this.onDragEnter)(e);
        return true;
    }
    return false;
}

operation Node.handleDragExit(e:CanvasDropEvent) {
    if (onDragExit <> null) {
        var pt = new <<java.awt.Point>>(e.x, e.y);
        ag.globalToLocal(pt);
        e.localX = pt.x;
        e.localY = pt.y;
        (this.onDragExit)(e);
        return true;
    }
    return false;
}

operation Node.onTransformChanged(t:AffineTransform) {
    if (this.tg <> null) {
/*
        if (alignmentTransform <> null) {
            t = new AffineTransform(t).concatenate(alignmentTransform);
        }
*/
        this.tg.setTransform(t);
    } 
}

LISTENER:F3NodeListener = new F3NodeListener();


trigger on Node.opacity = value {
    if (tg <> null) {
        fadeGroup.setAlpha(value);
    }
}

trigger on insert f into Node.filter {
    if (tg <> null) {
        filterGroup.setFilter(select i.getFilter() from i in filter);
    }
}

trigger on delete f from Node.filter {
    if (tg <> null) {
        filterGroup.setFilter(select i.getFilter() from i in filter);
    }
}

trigger on Node.filter[oldValue] = newValue {
    if (tg <> null) {
        filterGroup.setFilter(select i.getFilter() from i in filter);
    }
}

operation Node.getGlobalBounds() {
    return ag.getGlobalBounds();
}

trigger on Node.halign = value {
    if (value == null) {
        value = LEADING:HorizontalAlignment;
    }
    ag.setHorizontalAlignment(value.id);
}

trigger on Node.valign = value {
    if (value == null) {
        value = Top:VerticalAlignment;
    }
    ag.setHorizontalAlignment(value.id);
}

/*
trigger on Node._bounds[old] = value {

    var bounds = _bounds;
    var x = 0;
    var y = 0;
    if (valign <> null) {
        if (valign == TOP:VerticalAlignment) {
        } else if (valign == BOTTOM:VerticalAlignment) {
            y = -bounds.height;
            } else if (valign == CENTER:VerticalAlignment) {
            y = -bounds.height/2;
        }
    }
    if (halign <> null) {
        if (halign == LEADING:HorizontalAlignment) {
        } else if (halign == CENTER:HorizontalAlignment) {
            x = -bounds.width/2;
        } else if (halign == TRAILING:HorizontalAlignment) {
            x = -bounds.width;
        }
    }
    if (x <> 0 or y <> 0) {
        alignmentTransform = AffineTransform.getTranslateInstance(x, y);
        var t = alignmentTransform;
        if (affineTransform <> null) {
            t = new AffineTransform(affineTransform).concatenate(alignmentTransform);
        }
        tg.setTransform(t);
    }
}
*/

attribute Node.visible = true;

trigger on Node.visible = value {
    ag.setVisible(value);
}

trigger on Node.cursor[old] = value {
    if (hover) {
        this.getCanvas().zcanvas.setCursor(value.getCursor(), false);
    } else {
        this.installMouseListener();
    }
}


trigger on (Node.onMouseClicked = value) {
    this.installMouseListener();
}

trigger on (Node.onMouseEntered = value) {
    this.installMouseListener();
}

trigger on (Node.onMouseExited = value) {
    this.installMouseListener();
}

trigger on (Node.onMousePressed = value) {
    this.installMouseListener();
}

trigger on (Node.onMouseReleased = value) {
    this.installMouseListener();
}

trigger on (Node.onMouseDragged = value) {
    this.installMouseListener();
}

trigger on (Node.onMouseMoved = value) {
    this.installMouseListener();
}

trigger on Node.isSelectionRoot = value {
    if (value) {
        this.installMouseListener();
    }
}

attribute Node.selectable = true;

operation Node.getF3(obj:ZSceneGraphObject) {
    while (obj <> null) {
        var n = (Node)obj.getClientProperty("f3");
        if (n <> null) {
            return n;
        }
        if (obj instanceof ZNode) {
            obj = ((ZNode)obj).getParent();
        } else if (obj instanceof ZVisualComponent) {
            obj = ((ZVisualComponent)obj).getParents()[0];
        } else {
            obj = null;
        }
    }
    return null;
}

operation Node.installMouseMotionListener() {
    if (mouseMotionListener == null) {
        var self = this;
        selectable = true;
        operation setCursor() {
            getCanvas().zcanvas.setCursor(cursor.getCursor(), false);
        }
        mouseMotionListener = new ZMouseMotionListener() {
                operation mouseDragged(e:ZMouseEvent) {
                    if (self.exportDrag) {
                        return;
                    }
                    if (self.cursor <> null) {
                        setCursor();
                    }
                    e.percolate = self.onMouseDragged == null and self.onMouseMoved == null and not self.isSelectionRoot;
                    if (self.onMouseDragged <> null) {
                        var dragDeltaX = 0;
                        var dragDeltaY = 0;
                        var localDragDeltaX = 0;
                        var localDragDeltaY = 0;
                        var prev = if MOUSE_DRAG:ZMouseEvent <> null
                                        then MOUSE_DRAG:ZMouseEvent
                                    else MOUSE_PRESS:ZMouseEvent;
                        var screen = self.getCanvas().getComponent().getLocationOnScreen();
                        if (prev <> null) {
                           if (MOUSE_DRAG_SCREEN:Point == null) {
                               MOUSE_DRAG_SCREEN:Point = screen;
                           }
                           var screenDeltaX = screen.x - MOUSE_DRAG_SCREEN:Point.x;
                           var screenDeltaY = screen.y - MOUSE_DRAG_SCREEN:Point.y;
                           dragDeltaX = e.getX() - prev.getX() + screenDeltaX;
                           dragDeltaY = e.getY() - prev.getY() + screenDeltaY;
                           var pt1 = new <<java.awt.geom.Point2D$Double>>(prev.getX(), prev.getY());
                           var pt2 = new <<java.awt.geom.Point2D$Double>>(prev.getX()+dragDeltaX, prev.getY()+dragDeltaY);
                           self.tg.globalToLocal(pt1);
                           self.tg.globalToLocal(pt2);
                           localDragDeltaX = pt2.getX() - pt1.getX();
                           localDragDeltaY = pt2.getY() - pt1.getY();
                           
                        }
                        MOUSE_DRAG:ZMouseEvent = e;
                        MOUSE_DRAG_SCREEN:Point = screen;
                        var event = self.makeCanvasMouseEvent(e);
                        event.dragTranslation = {x: dragDeltaX, y: dragDeltaY};
                        event.localDragTranslation = {x: localDragDeltaX, y: localDragDeltaY};
                        (self.onMouseDragged)(event);
                    }
                }
                operation mouseMoved(e:ZMouseEvent) {
                    if (self.cursor <> null) {
                        setCursor();
                    }
                    e.percolate = self.onMouseMoved == null and self.onMouseDragged == null and not self.isSelectionRoot;
                    if (self.onMouseMoved <> null) {
                        (self.onMouseMoved)(self.makeCanvasMouseEvent(e));
                    }
                }
            };
        if (tg <> null) {
            tg.addMouseMotionListener(mouseMotionListener);
        }
    }
}


operation Node.makeCanvasMouseEvent(e:ZMouseEvent) {
    var pt = new <<java.awt.geom.Point2D$Double>>(e.getX(), e.getY());
    var scale = tg.globalToLocal(pt);
    var result = CanvasMouseEvent {
        modifiers: [if e.isAltDown() then ALT else null,
                    if e.isControlDown() then CTRL else null, 
                    if e.isMetaDown() then META else null,
                    if e.isShiftDown() then SHIFT else null]
        button: if SwingUtilities.isLeftMouseButton(e) then 1 else if SwingUtilities.isRightMouseButton(e) then 3 else 2
        clickCount: e.getClickCount()
        localX: pt.getX()
        localY: pt.getY()
        x: e.getX()
        y: e.getY()
        percolate: e.percolate
        source: e
    };
    return result;
}

trigger on Node.selectable = value {
    sg.setPickable(value);
    tg.setPickable(value);
    ag.setPickable(value);
}

operation Node.installMouseListener() {
    if (mouseListener == null) {
        var self = this;
        selectable = true;
        var entered  = false;
        operation setCursor() {
            getCanvas().zcanvas.setCursor(cursor.getCursor(), false);
        }
        mouseListener = new ZMouseListener() {

                operation mouseClicked(e:ZMouseEvent) {
                    if (self.focusable) {
                        self.focused = true;
                    }
                    if (self.onMouseClicked <> null) {
                        e.percolate = false;
                        (self.onMouseClicked)(self.makeCanvasMouseEvent(e));
                    } else if (self.onMousePressed <> null) {
                        e.percolate = false;
                    } else {
                        e.percolate = not self.isSelectionRoot;
                    }
                }

                operation mouseEntered(e:ZMouseEvent) {
                    if (self.cursor <> null) {
                        setCursor();        
                    } 
                    if (self.onMouseEntered <> null) {
                        (self.onMouseEntered)(self.makeCanvasMouseEvent(e));
                    }
                    e.percolate = not self.isSelectionRoot;
                }

                operation mouseExited(e:ZMouseEvent) {
                    if (self.onMouseExited <> null) {
                        (self.onMouseExited)(self.makeCanvasMouseEvent(e));
                    } 
                    if (self.cursor <> null) {
                        self.getCanvas().zcanvas.resetCursor();
                    }
                    e.percolate = not self.isSelectionRoot;
                }

                operation mousePressed(e:ZMouseEvent) {
                    MOUSE_PRESS:ZMouseEvent = e;        
                    var c = self.getCanvas();
                    MOUSE_DRAG_SCREEN:Point = c.getComponent().getLocationOnScreen();

                    if (c.focusable) {
                        c.zcanvas.requestFocus();
                    }
                    if (self.onMousePressed <> null) {
                        e.percolate = false;
                        (self.onMousePressed)(self.makeCanvasMouseEvent(e));
                    } else if (self.onMouseClicked <> null) {
                        e.percolate = false;
                    } else {
                        e.percolate = not self.isSelectionRoot;
                    }
                }

                operation mouseReleased(e:ZMouseEvent) {
                    MOUSE_DRAG:ZMouseEvent = null;
                    MOUSE_PRESS:ZMouseEvent = null;
                    MOUSE_DRAG_SCREEN:Point = null;
                    if (self.onMouseReleased <> null) {
                        e.percolate = false;
                        (self.onMouseReleased)(self.makeCanvasMouseEvent(e));
                    } else if (self.onMousePressed <> null or self.onMouseClicked <> null) {
                        e.percolate = false;
                    } else {
                        e.percolate = not self.isSelectionRoot;
                    }
                }
            };
        if (tg <> null) {
            tg.addMouseListener(mouseListener);
        }
    }
    this.installMouseMotionListener();
}

operation Node.getNode() {
    if (tg == null) {
        var canvas = this.getCanvas();
        this.onSetCanvas(canvas);
        if (focused) {
            canvas.focusedNode = this;
        }
        sg = this.createNode();
        if (sg == null) {
            throw "create node for class {this.class.Name} returned null";
        }
        //sg.putClientProperty("f3", this);
        filterGroup = new ZFilterGroup();
        filterGroup.addChild(sg);
        fadeGroup = new ZFadeGroup();
        fadeGroup.addChild(filterGroup);
        cg = new ZClipGroup();
        cg.setClipVisible(false);
        cg.addChild(fadeGroup);
        tg = new ZTransformGroup();
        tg.addChild(cg);
        tg.setChildrenPickable(true);
        tg.putClientProperty("f3", this);
        tg.addNodeListener(LISTENER:F3NodeListener);
        ag = new ZAlignmentGroup();
        if (halign <> null) {
            ag.setHorizontalAlignment(halign.id);
        }
        if (valign <> null) {
            ag.setVerticalAlignment(valign.id);
        }
        ag.addChild(tg);
        if (clipNode <> null) {
            cg.setClipNode(clipNode.getNode());
        }
        if (antialiasClip <> null) {
             cg.setSmooth(antialiasClip);
        }
        if (affineTransform <> null) {
            tg.setTransform(affineTransform);
        }
        if (mouseListener <> null) {
            ag.addMouseListener(mouseListener);
        }
        if (mouseMotionListener <> null) {
            ag.addMouseMotionListener(mouseMotionListener);
        }
        //sg.setPickable(selectable);
        ag.setPickable(selectable);
        ag.setVisible(visible);
        if (scaleToFitCanvas) {
            insert this into canvas.scaleToFitList;
        }
        if (this instanceof SizeableCanvasElement) { // hack
            var r = (SizeableCanvasElement)this;
            if (r.sizeToFitCanvas) {
                insert r into canvas.sizeToFitList;
            }
        }
        if (opacity <> null and opacity <> 1.0) {
            fadeGroup.setAlpha(opacity);
        }
        if (filter <> null) {
           filterGroup.setFilter(select i.getFilter() from i in filter);
        }
        if (toolTipText <> null) {
            ag.setToolTipText(toolTipText);
        }
        if (id <> null) {
            ag.putClientProperty("id", id);
        }
    }
    return ag;
}

trigger on Node.toolTipText = value {
    if (ag <> null) {
        ag.setToolTipText(value);
    }
}


trigger on Node.scaleToFitCanvas = value {
    if (this.parentCanvasElement <> null) {
        var canvas = this.getCanvas();
        if (value) {
            insert this into canvas.scaleToFitList;
        } else {
            delete canvas.scaleToFitList[. == this];
        }
    }
}

operation Node.getCanvas() {
    var n = parentCanvasElement;
    while (n <> null) {
        if (n instanceof Canvas) {
            cachedCanvas = (Canvas)n;
            return cachedCanvas;
        }
        n = n.parentCanvasElement;
    }
    return cachedCanvas;
}

trigger on Node.focused = value {
    var canvas = this.getCanvas();
    if (value) {
        canvas.focusedNode = this;
    } else {
        if (canvas.focusedNode == this) {
            canvas.focusedNode = null;
        }
    }
}

trigger on Node.dragCount = value {
    if (value > 0) {
        this.doDragExport();
    }
}

operation Node.doDragExport() {
    var e = (this.exportAsDrag)();
    var c = this.getCanvas();
    c.doExportAsDrag(e);
}
