/*
 *  $Id$
 * 
 *  Copyright 2007 Sun Microsystems, Inc. All rights reserved.
 *  SUN PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

package javafx.ui.canvas;
import javafx.ui.canvas.VisualNode;
import net.java.javafx.jazz.component.ZText;
import java.awt.Font as AWTFont;
import javafx.ui.Font;
import javafx.ui.Alignment;
/**
 * A canvas node that displays text defined by a location (x, y),
 * a string of characters (content), and a font. If stroking and filling
 * and/or shape operations aren't required, it's often more useful to 
 * use a View with a Label or SimpleLabel to represent text (for example,
 * in that case you can use HTML markup).
 */

public class Text extends Shape {
    // private:
    private attribute ztext: ZText;
    // public
    /** Determines whether this text can be edited. Defaults to false. */
    public attribute editable: Boolean;
    /** The character content of this text. */
    public attribute content: String;
    /** The x coordinate of the location of this text. */
    public attribute x: Number;
    /** The y coordinate of the location of this text. */
    public attribute y: Number;
    /** The font used to render the characters of this text. */
    public attribute font: Font?;
    protected attribute awtFont: AWTFont?;

    public attribute alignment: Alignment;
}

attribute Text.alignment = LEADING;
attribute Text.awtFont = bind font.getFont();

trigger on Text.awtFont = value {
    ztext.setFont(value);
}

trigger on Text.content = value {
    ztext.setText(value);
}

trigger on Text.x = value {
    ztext.setTranslateX(x);
}

trigger on Text.y = value {
    ztext.setTranslateY(y);
}

operation Text.createShape() {
    ztext = new ZText(content);
    ztext.setEditable(editable);
    ztext.setTranslateX(x);
    ztext.setTranslateY(y);
    ztext.setFont(awtFont);
    return ztext;
}

trigger on Text.content = value {
    ztext.setText(value);
}

attribute Text.fill = black;
attribute Text.stroke = null;
