/*
 *  $Id$
 * 
 *  Copyright 2007 Sun Microsystems, Inc. All rights reserved.
 *  SUN PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

package javafx.ui.canvas;
import java.awt.geom.AffineTransform;
import java.lang.Math;

/**
 * Interface for operations that perform transformations on graphics objects.
 */

public class Transform {
    protected attribute transformable: Transformable?;
    protected attribute transform: AffineTransform?;
    ...
}

trigger on Transform.transform = value {
    transformable.updateTransform();
}

translate:Transform = function(x:Number, y:Number) {
    return translate:Translate(x, y);
};

matrix:Transform = operation(a:Number,b:Number,c:Number,d:Number,e:Number,f:Number) {
    return matrix:Matrix(a,b,c,d,e,f);
};

rotate:Transform = function(z:Number, x:Number, y:Number) {
    return rotate:Rotate(z, x, y);
};

scale:Transform = function(x:Number, y:Number) {
    return scale:Scale(x, y);
};

scaleX:Transform = operation(x: Number) {return scaleX:Scale(x);};
scaleY:Transform = operation(y: Number) {return scaleY:Scale(y);};

skew:Transform = operation(x:Number, y:Number) {return skew:Skew(x, y);};
skewX:Transform = operation(x:Number) {return skewX:Skew(x);};
skewY:Transform = operation(y:Number) {return skewY:Skew(y);};

translateY:Transform = operation(y:Number) {return translateY:Translate(y);};
translateX:Transform = operation(x:Number) {return translateX:Translate(x);};
scaleXY:Transform = operation(s:Number) {return scaleXY:Scale(s);};

//rotateY:Transform = function(angle, x, y, z, f) = rotateY:RotateZ(angle, x,  y, z, f);
//rotateX:Transform = function(angle, x, y, z, f) = rotateX:RotateZ(angle, x,  y, z, f);

public class CompositeTransform extends Transform {
    public attribute transforms: Transform*;
    private attribute txs: AffineTransform*;
    private operation updateTransform();
}

attribute CompositeTransform.txs = bind select t.transform from t in transforms;

operation CompositeTransform.updateTransform() {
    var result = new AffineTransform();
    for (t in txs) {
        result.concatenate(t);
    }
    transform = result;
}

trigger on insert t into CompositeTransform.txs {
    updateTransform();
}

trigger on delete t from CompositeTransform.txs {
    updateTransform();
}

trigger on CompositeTransform.txs[oldValue] = newValue {
    updateTransform();
}
