/*
 *  $Id$
 * 
 *  Copyright 2007 Sun Microsystems, Inc. All rights reserved.
 *  SUN PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

package javafx.ui.canvas;
import java.awt.geom.AffineTransform;
/** 
 * Interface for graphic objects that can be transformed (translated,
 * scaled, skewed, or rotated).
 */
public class Transformable {
    /** A list of transformation operations that will be performed on this object. */
    public attribute transform: Transform*;
    protected attribute affineTransform: AffineTransform?;
    protected operation updateTransform();
    protected operation onTransformChanged(t:AffineTransform);
    ...
}

operation Transformable.updateTransform() {
    var t = new AffineTransform();
    for (i in transform) {
        if (i.transform <> null) {
            t.concatenate(i.transform);
        }
    }
    if (affineTransform <> t) {
        affineTransform = t;
        if (affineTransform == t) {
            this.onTransformChanged(affineTransform);
        }
    }
}

trigger on insert t into Transformable.transform {
    t.transformable = this;
    this.updateTransform();
}

trigger on Transformable.transform[oldValue] = newValue {
    if (oldValue.transformable == this) {
        oldValue.transformable = null;
    }
    newValue.transformable = this;
    this.updateTransform();
}

trigger on delete t from Transformable.transform {
    if (t.transformable == this) {
        t.transformable = null;
    }
    this.updateTransform();
}
